# SPDX-FileCopyrightText: © 2025 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0

"""
This file contains functions used to do a PCIe level reset for Blackhole or Wormhole chip.
"""

import os
import sys
import time
import fcntl
import struct
import glob
from enum import IntEnum
from typing import List
from pyluwen import PciChip
from tt_tools_common.ui_common.themes import CMD_LINE_COLOR
from tt_tools_common.utils_common.system_utils import (
    check_driver_version,
    get_host_info,
)

class IoctlResetFlags(IntEnum):
    RESTORE_STATE = 0
    RESET_PCIE_LINK = 1
    CONFIG_WRITE = 2
    USER_RESET = 3
    ASIC_RESET = 4
    ASIC_DMC_RESET = 5
    POST_RESET = 6

class ChipReset:
    """Class to perform a chip-level reset on WH and BH PCIe boards"""

    # magic numbers for reset
    TENSTORRENT_IOCTL_MAGIC = 0xFA
    TENSTORRENT_IOCTL_RESET_DEVICE = (TENSTORRENT_IOCTL_MAGIC << 8) | 6

    def reset_device_ioctl(self, interface_id: int, flags: int) -> bool:
        dev_path = f"/dev/tenstorrent/{interface_id}"
        dev_fd = os.open(
            dev_path, os.O_RDWR | os.O_CLOEXEC
        )  # Raises FileNotFoundError and other appropriate exceptions.
        try:
            reset_device_in_struct = "II"
            reset_device_out_struct = "II"
            reset_device_struct = reset_device_in_struct + reset_device_out_struct

            input_size_bytes = struct.calcsize(reset_device_in_struct)
            output_size_bytes = struct.calcsize(reset_device_out_struct)

            reset_device_buf = bytearray(
                struct.pack(reset_device_struct, output_size_bytes, flags, 0, 0)
            )
            fcntl.ioctl(
                dev_fd, self.TENSTORRENT_IOCTL_RESET_DEVICE, reset_device_buf
            )  # Raises OSError

            output_buf = reset_device_buf[input_size_bytes:]
            _, result = struct.unpack(reset_device_out_struct, output_buf)

            return result == 0
        finally:
            os.close(dev_fd)

    def wait_for_device_to_reappear(self, bdf: str, timeout: int = 10) -> int:
        print(
            CMD_LINE_COLOR.BLUE,
            f"Waiting for devices to reappear on pci bus...",
            CMD_LINE_COLOR.ENDC,)
        deadline = time.time() + timeout
        device_reappeared = False

        while time.time() < deadline and not device_reappeared:
            matches = glob.glob(f"/sys/bus/pci/devices/{bdf}/tenstorrent/tenstorrent!*")
            if matches:
                interface_id = int(os.path.basename(matches[0]).replace("tenstorrent!", ""))
                dev_path = f"/dev/tenstorrent/{interface_id}"
                if os.path.exists(dev_path):
                    device_reappeared = True

        if not device_reappeared:
            print(
                CMD_LINE_COLOR.RED,
                f"Timeout waiting for device at PCI index {interface_id} to reappear.",
                CMD_LINE_COLOR.ENDC,
            )
            sys.exit(1)

        return interface_id


    def full_lds_reset(
        self, pci_interfaces: List[int], reset_m3: bool = False, silent: bool = False
    ) -> List[PciChip]:
        """Performs a full LDS reset of a list of chips"""

        # Check the driver version and bail if reset cannot be supported
        check_driver_version(operation="reset", minimum_required_version_str="2.4.1")

        # Due to how Arm systems deal with PCIe device rescans, WH device resets don't work on that platform.
        # Check for platform and bail if it's Arm
        platform = get_host_info()["Platform"]
        if platform.startswith("arm") or platform.startswith("aarch"):
            print(
                CMD_LINE_COLOR.RED,
                "Cannot perform board reset on Arm systems, please reboot the system to reset the boards. Exiting...",
                CMD_LINE_COLOR.ENDC,
            )
            sys.exit(1)

        # Remove duplicates from the input list of PCI interfaces
        pci_interfaces = list(set(pci_interfaces))
        if not silent:
            print(
                f"{CMD_LINE_COLOR.BLUE} Starting reset on devices at PCI indices: {str(pci_interfaces)[1:-1]} {CMD_LINE_COLOR.ENDC}"
            )

        bdf_list = []

        for pci_interface in pci_interfaces:
            chip = PciChip(pci_interface=pci_interface)
            bdf = chip.get_pci_bdf()
            bdf_list.append(bdf)
        
        for pci_interface in pci_interfaces:
            if not self.reset_device_ioctl(pci_interface, IoctlResetFlags.RESET_PCIE_LINK):
                print(
                    CMD_LINE_COLOR.YELLOW,
                    f"Warning: Secondary bus reset not completed for device at PCI index {pci_interface}. Continuing with reset.",
                    CMD_LINE_COLOR.ENDC
                )


        for pci_interface in pci_interfaces:
            if reset_m3:
                reset_flag = IoctlResetFlags.ASIC_DMC_RESET
            else:
                reset_flag = IoctlResetFlags.ASIC_RESET

            self.reset_device_ioctl(pci_interface, reset_flag)

        post_reset_wait = 20 if reset_m3 else max(2, 0.4 * len(pci_interfaces))
        print(
            CMD_LINE_COLOR.BLUE,
            f"Waiting for {post_reset_wait} seconds for potential hotplug removal.",
            CMD_LINE_COLOR.ENDC
        )
        time.sleep(post_reset_wait)

        for pci_interface,bdf in zip(pci_interfaces,bdf_list):
            new_id = self.wait_for_device_to_reappear(bdf) 

            if self.reset_device_ioctl(new_id, IoctlResetFlags.POST_RESET):
                print(
                    f"{CMD_LINE_COLOR.GREEN} Reset successfully completed for device at PCI index {pci_interface}. {CMD_LINE_COLOR.ENDC}"
                )
            else:
                print(
                    f"{CMD_LINE_COLOR.RED} Post-reset actions did not complete successfully for device at PCI index {pci_interface}. {CMD_LINE_COLOR.ENDC}"
                )
                sys.exit(1)
        
        # All went well print success message
        # other sanity checks go here
        if not silent:
            print(
                f"{CMD_LINE_COLOR.BLUE} Finishing reset on devices at PCI indices: {str(pci_interfaces)[1:-1]} {CMD_LINE_COLOR.ENDC}"
            )

        pci_chips = [PciChip(pci_interface=interface) for interface in pci_interfaces]
        return pci_chips



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0

"""
This file contains functions used to do a PCIe level reset for Wormhole chip.
"""

import os
import sys
import time
import fcntl
import struct
from typing import List
from pyluwen import PciChip
from tt_tools_common.ui_common.themes import CMD_LINE_COLOR
from tt_tools_common.utils_common.tools_utils import read_refclk_counter
from tt_tools_common.utils_common.system_utils import (
    check_driver_version,
    get_host_info,
    is_driver_version_at_least,
    get_driver_version
)
from tt_tools_common.reset_common.chip_reset import ChipReset


class WHChipReset:
    """Class to perform a chip level reset on WH PCIe boards"""

    # WH magic numbers for reset
    TENSTORRENT_IOCTL_MAGIC = 0xFA
    TENSTORRENT_IOCTL_RESET_DEVICE = (TENSTORRENT_IOCTL_MAGIC << 8) | 6
    TENSTORRENT_RESET_DEVICE_RESTORE_STATE = 0
    TENSTORRENT_RESET_DEVICE_RESET_PCIE_LINK = 1
    A3_STATE_PROP_TIME = 0.03
    POST_RESET_MSG_WAIT_TIME = 2
    MSG_TRIGGER_SPI_COPY_LtoR = 0x50
    MSG_TYPE_ARC_STATE3 = 0xA3
    MSG_TYPE_TRIGGER_RESET = 0x56

    def reset_device_ioctl(self, interface_id: int, flags: int) -> bool:
        dev_path = f"/dev/tenstorrent/{interface_id}"
        dev_fd = os.open(
            dev_path, os.O_RDWR | os.O_CLOEXEC
        )  # Raises FileNotFoundError and other appropriate exceptions.
        try:
            reset_device_in_struct = "II"
            reset_device_out_struct = "II"
            reset_device_struct = reset_device_in_struct + reset_device_out_struct

            input_size_bytes = struct.calcsize(reset_device_in_struct)
            output_size_bytes = struct.calcsize(reset_device_out_struct)
            reset_device_buf = bytearray(
                struct.pack(reset_device_struct, output_size_bytes, flags, 0, 0)
            )
            fcntl.ioctl(
                dev_fd, self.TENSTORRENT_IOCTL_RESET_DEVICE, reset_device_buf
            )  # Raises OSError

            output_buf = reset_device_buf[input_size_bytes:]
            _, result = struct.unpack(reset_device_out_struct, output_buf)

            return result == 0
        finally:
            os.close(dev_fd)

    def full_lds_reset(
        self, pci_interfaces: List[int], reset_m3: bool = False, silent: bool = False
    ) -> List[PciChip]:
        """Performs a full LDS reset of a list of chips"""
        
        # Use new reset for driver version >= 2.4.1
        if is_driver_version_at_least(get_driver_version(), "2.4.1"):
            return ChipReset().full_lds_reset(pci_interfaces, reset_m3, silent)

        if not silent:
            print(
                CMD_LINE_COLOR.YELLOW,
                "Notice: Using legacy WH reset implementation. This will be removed in a later version.",
                "Please upgrade tt-kmd to version 2.4.1 or newer to use the updated reset sequence.",
                CMD_LINE_COLOR.ENDC
            )

        # Check the driver version and bail if link reset cannot be supported
        check_driver_version(operation="board reset")

        # Due to how Arm systems deal with PCIe device rescans, WH device resets don't work on that platform.
        # Check for platform and bail if it's Arm
        platform = get_host_info()["Platform"]
        if platform.startswith("arm") or platform.startswith("aarch"):
            print(
                CMD_LINE_COLOR.RED,
                "Cannot perform WH board reset on Arm systems, please reboot the system to reset the boards. Exiting...",
                CMD_LINE_COLOR.ENDC,
            )
            sys.exit(1)

        # Remove duplicates from the input list of PCI interfaces
        pci_interfaces = list(set(pci_interfaces))
        if not silent:
            print(
                f"{CMD_LINE_COLOR.BLUE} Starting PCI link reset on WH devices at PCI indices: {str(pci_interfaces)[1:-1]} {CMD_LINE_COLOR.ENDC}"
            )

        for pci_interface in pci_interfaces:
            self.reset_device_ioctl(
                pci_interface, self.TENSTORRENT_RESET_DEVICE_RESET_PCIE_LINK
            )
        pci_chips = [PciChip(pci_interface=interface) for interface in pci_interfaces]
        refclk_list = []
        fail = False
        # Trigger resets for all chips in order
        for chip in pci_chips:
            # Collect the arc refclk for the chip before sending reset arc messages
            try:
                refclk_list.append(read_refclk_counter(chip))
            except Exception as e:
                # If we get to this point means ioctl reset isn't enough to reset the chip
                # This is a fatal error, we should exit and recommend user to reboot the system
                print(
                    CMD_LINE_COLOR.RED,
                    "Failed to recover WH chip, please reboot the system to reset the chip. Exiting...",
                    CMD_LINE_COLOR.ENDC,
                )
                sys.exit(1)
            # Trigger A3 safe state. A3 is a safe state where there are no more pending regulator requests.
            chip.arc_msg(self.MSG_TYPE_ARC_STATE3, wait_for_done=True)
            time.sleep(self.A3_STATE_PROP_TIME)
            # Triggers M3 board level reset by sending arc msg.
            if reset_m3:
                chip.arc_msg(self.MSG_TYPE_TRIGGER_RESET, wait_for_done=False, arg0=3)
            else:
                chip.arc_msg(self.MSG_TYPE_TRIGGER_RESET, wait_for_done=False)

        time.sleep(self.POST_RESET_MSG_WAIT_TIME)

        for i, (chip, pci_interface) in enumerate(zip(pci_chips, pci_interfaces)):
            self.reset_device_ioctl(
                pci_interface, self.TENSTORRENT_RESET_DEVICE_RESTORE_STATE
            )
            current_refclk = read_refclk_counter(chip)
            if refclk_list[i] < current_refclk:
                print(
                    CMD_LINE_COLOR.RED,
                    f"Reset for PCI {pci_interface} didn't go through! Refclk didn't reset. Value before: {refclk_list[i]}, value after: {current_refclk}",
                    CMD_LINE_COLOR.ENDC,
                )
                fail = True

        if fail:
            print(
                CMD_LINE_COLOR.YELLOW,
                "Reset failed for one or more boards, returning with non-zero exit code",
                CMD_LINE_COLOR.ENDC,
            )
            sys.exit(1)
        else:
            #  All went well print success message
            if not silent:
                print(
                    f"{CMD_LINE_COLOR.GREEN} Finishing PCI link reset on WH devices at PCI indices: {str(pci_interfaces)[1:-1]} {CMD_LINE_COLOR.ENDC}"
                )

        return pci_chips



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0

"""
This file contains common utilities used by all tt-tools.
"""
import os
import sys
import time
from typing import List
import importlib.resources
from yaml import safe_load
from pyluwen import PciChip, detect_chips_fallible
from tt_tools_common.ui_common.themes import CMD_LINE_COLOR


def init_fw_defines(chip_name: str = "wormhole", tool_name: str = "tt_smi"):
    """
    Loads the fw_defines.yaml with arc msg definitions from the chip's data directory.
    """
    fw_defines = safe_load(
        get_chip_data(chip_name, "fw_defines.yaml", False, tool_name)
    )
    return fw_defines


def int_to_bits(x):
    return list(filter(lambda b: x & (1 << b), range(x.bit_length())))


def get_chip_data(chip_name, file, internal: bool, tool_name="tt_smi"):
    """
    Helper function to load a file from the chip's data directory.
    """
    with importlib.resources.path(f"{tool_name}", "") as path:
        if chip_name not in ["wormhole", "grayskull"]:
            raise Exception("Only support fw messages for Wh or GS chips")
        prefix = chip_name
        if internal:
            prefix = f".ignored/{prefix}"
        else:
            prefix = f"data/{prefix}" if not ".data" in tool_name else prefix
        return open(str(path.joinpath(f"{prefix}/{file}")))


def get_board_type(board_id: str) -> str:
    """
    Get board type from board ID string.
    Ex:
        Board ID: AA-BBBBB-C-D-EE-FF-XXX
                   ^     ^ ^ ^  ^  ^   ^
                   |     | | |  |  |   +- XXX
                   |     | | |  |  +----- FF
                   |     | | |  +-------- EE
                   |     | | +----------- D
                   |     | +------------- C = Revision
                   |     +--------------- BBBBB = Unique Part Identifier (UPI)
                   +--------------------- AA
    """
    serial_num = int(f"0x{board_id}", base=16)
    upi = (serial_num >> 36) & 0xFFFFF
    rev = (serial_num >> 32) & 0xF

    if upi == 0x1:
        if rev == 0x2:
            return "E300_R2"
        elif rev in (0x3, 0x4):
            return "E300_R3"
        else:
            return "N/A"
    elif upi == 0x3:
        # Formerly E300_105
        return "e150"
    elif upi == 0x7:
        return "e75"
    elif upi == 0x8:
        return "NEBULA_CB"
    elif upi == 0xA:
        # Formerly E300_X2
        return "e300"
    elif upi == 0xB:
        return "GALAXY"
    elif upi == 0x14:
        # Formerly NEBULA_X2
        return "n300"
    elif upi == 0x18:
        # Formerly NEBULA_X1
        return "n150"
    else:
        return "N/A"


def hex_to_date(hexdate: int, include_time=True):
    """Converts a date given in hex from format 0xYMDDHHMM to string YYYY-MM-DD HH:MM"""
    if hexdate == 0 or hexdate == 0xFFFFFFFF:
        return "N/A"

    year = (hexdate >> 28 & 0xF) + 2020
    month = hexdate >> 24 & 0xF
    day = hexdate >> 16 & 0xFF
    hour = hexdate >> 8 & 0xFF
    minute = hexdate & 0xFF

    date = f"{year:04}-{month:02}-{day:02}"

    if include_time:
        date += f" {hour:02}:{minute:02}"

    return date


def init_logging(log_folder: str):
    """Create tt-mod log folders if they don't exist"""
    if not os.path.isdir(log_folder):
        os.mkdir(log_folder)


def semver_to_hex(semver: str):
    """Converts a semantic version string from format 10.15.1 to hex 0x0A0F0100"""
    major, minor, patch = semver.split(".")
    byte_array = bytearray([0, int(major), int(minor), int(patch)])
    return f"{int.from_bytes(byte_array, byteorder='big'):08x}"


def date_to_hex(date: int):
    """Converts a given date string from format YYYYMMDDHHMM to hex 0xYMDDHHMM"""
    year = int(date[0:4]) - 2020
    month = int(date[4:6])
    day = int(date[6:8])
    hour = int(date[8:10])
    minute = int(date[10:12])
    byte_array = bytearray([year * 16 + month, day, hour, minute])
    return f"{int.from_bytes(byte_array, byteorder='big'):08x}"


def hex_to_semver(hexsemver: int):
    """Converts a semantic version string from format 0x0A0F0100 to 10.15.1"""
    if hexsemver == 0 or hexsemver == 0xFFFFFFFF:
        raise ValueError("hexsemver is invalid!")

    major = hexsemver >> 16 & 0xFF
    minor = hexsemver >> 8 & 0xFF
    patch = hexsemver >> 0 & 0xFF

    return f"{major}.{minor}.{patch}"


def hex_to_semver_eth(hexsemver: int):
    """Converts a semantic version string from format 0x061000 to 6.1.0"""
    if hexsemver == 0 or hexsemver == 0xFFFFFF:
        return "N/A"

    major = hexsemver >> 16 & 0xFF
    minor = hexsemver >> 12 & 0xF
    patch = hexsemver & 0xFFF

    return f"{major}.{minor}.{patch}"


def hex_to_semver_m3_fw(hexsemver: int):
    """Converts a semantic version string from format 0x0A0F0100 to 10.15.1"""
    if hexsemver == 0 or hexsemver == 0xFFFFFFFF:
        return "N/A"

    major = hexsemver >> 24 & 0xFF
    minor = hexsemver >> 16 & 0xFF
    patch = hexsemver >> 8 & 0xFF
    ver = hexsemver >> 0 & 0xFF

    return f"{major}.{minor}.{patch}.{ver}"

def hex_to_semver_zephyr_fw(hexsemver: int):
    """Converts a semantic version string from format 0x0A0F0100 to 10.15.1 or
    0x0A0F0101 to 10.15.1-rc1"""
    if hexsemver == 0 or hexsemver == 0xFFFFFFFF:
        return "N/A"

    major = hexsemver >> 24 & 0xFF
    minor = hexsemver >> 16 & 0xFF
    patch = hexsemver >> 8 & 0xFF
    rc_num = hexsemver >> 0 & 0xFF

    rc = ""
    # rc of 0 indicates a final release and requires no suffix
    # rc > 0 indicates a release candidate (i.e. -rc1, -rc2, etc)
    if rc_num > 0:
        rc = f"-rc{rc_num}"

    return f"{major}.{minor}.{patch}{rc}"


def init_logging(log_folder: str):
    """Create log folders if they don't exist"""
    if not os.path.isdir(log_folder):
        os.makedirs(log_folder)


# Show that the refclock counter (ARC_RESET.REFCLK_COUNTER_LOW/HIGH) is ticking at
# the expected frequency, independent of ARCLK.


def read_refclk_counter(chip) -> int:
    if chip.as_gs():
        return None
    high_addr = chip.axi_translate("ARC_RESET.REFCLK_COUNTER_HIGH").addr
    low_addr = chip.axi_translate("ARC_RESET.REFCLK_COUNTER_LOW").addr
    high1 = chip.axi_read32(high_addr)
    low = chip.axi_read32(low_addr)
    high2 = chip.axi_read32(high_addr)

    if high1 != high2:
        low = chip.axi_read32(low_addr)

    return (high2 << 32) | low


# Returns REFCLK_COUNTER rate in MHz
def refclk_counter_rate(chip, delay_interval: float = 0.1) -> float:
    before_refclk = read_refclk_counter(chip)
    before_ns = time.time_ns()

    time.sleep(delay_interval)

    after_refclk = read_refclk_counter(chip)
    after_ns = time.time_ns()

    return (after_refclk - before_refclk) * 1000 / (after_ns - before_ns)


def check_refclk_counter_read_speed(chip):
    loops = 100

    before_ns = time.time_ns()
    for _ in range(loops):
        read_refclk_counter(chip)
    after_ns = time.time_ns()

    if after_ns - before_ns > 100_000 * loops:
        us_per = (after_ns - before_ns) // (loops * 1000)
        print(f"REFCLK_COUNTER reads are unusually slow ({us_per}us).")


def check_refclk_counter_rate(chip, expected_refclk: float, accuracy: float):
    observed_refclk = refclk_counter_rate(chip)
    # print(f"refclk {observed_refclk}MHz")
    if (
        abs(expected_refclk - observed_refclk) / min(expected_refclk, observed_refclk)
        > accuracy / 100
    ):
        return f"REFCLK_COUNTER outside of allowed range: {observed_refclk}"
    else:
        return None


def detect_chips_with_callback(
    local_only: bool = False,
    ignore_ethernet: bool = False,
    print_status: bool = True,
) -> List[PciChip]:
    """
    This will create a chip which only guarantees that you have communication with the chip.
    """

    chip_count = 0
    block_count = 0
    arc_count = 0
    dram_count = 0
    spinner = ["\\", "|", "/", "-"]

    last_draw = time.time()

    def chip_detect_callback(status):
        nonlocal chip_count, last_draw, block_count, arc_count, dram_count

        if status.new_chip():
            chip_count += 1
        elif status.correct_down():
            chip_count -= 1
        chip_count = max(chip_count, 0)

        # Move the cursor and delete the previous block of printed lines
        if block_count > 0 and print_status:
            print(f"\033[{block_count}A", end="", flush=True)
            print(f"\033[J", end="", flush=True)

        if print_status:
            print(
                f"\r{CMD_LINE_COLOR.PURPLE} Detected Chips: {CMD_LINE_COLOR.YELLOW}{chip_count}{CMD_LINE_COLOR.ENDC}\n",
                end="",
                flush=True,
            )
            block_count = 1

        # Prune and update the status string
        status_string = status.status_string()
        if status_string is not None and local_only is False and print_status is True:
            # remove empty lines
            for line in list(filter(None, status_string.splitlines())):
                # Up the counter for each line printed
                block_count += 1
                if "ARC" in line:
                    arc_count = arc_count + 1
                    # Spinner character is based on the number of ARCs detected
                    print(
                        f"\r{CMD_LINE_COLOR.BLUE} Detecting ARC: "
                        + f"{CMD_LINE_COLOR.YELLOW}{spinner[arc_count % len(spinner)]}{CMD_LINE_COLOR.ENDC}",
                        flush=True,
                    )
                elif "DRAM" in line:
                    dram_count = dram_count + 1
                    print(
                        f"\r{CMD_LINE_COLOR.BLUE} Detecting DRAM: "
                        + f"{CMD_LINE_COLOR.YELLOW}{spinner[dram_count % len(spinner)]}{CMD_LINE_COLOR.ENDC}",
                        flush=True,
                    )
                elif "ETH" in line:
                    import re

                    paren_pattern = r"\((.*?)\)"
                    bracket_pattern = r"\[(.*?)\]"
                    # Find substrings between parentheses
                    paren_substr = re.search(paren_pattern, line)
                    bracket_substr = re.search(bracket_pattern, line)
                    # Extract substring from matched group
                    bracket_substr = bracket_substr.group(1) if bracket_substr else ""
                    paren_substr = paren_substr.group(1) if paren_substr else ""
                    line = re.sub(paren_pattern, "", line)
                    print(
                        f"\r {CMD_LINE_COLOR.PURPLE}[{paren_substr}]{CMD_LINE_COLOR.BLUE}{line}: "
                        + f"{CMD_LINE_COLOR.YELLOW}{spinner[dram_count % len(spinner)]}{CMD_LINE_COLOR.ENDC}",
                        flush=True,
                    )
                else:
                    print(
                        f"\r{line}",
                        flush=True,
                    )

    output = []
    for device in detect_chips_fallible(
        local_only=local_only,
        continue_on_failure=False,
        callback=chip_detect_callback,
        noc_safe=ignore_ethernet,
    ):
        if not device.have_comms():
            raise Exception(
                f"Do not have communication with {device}, you should reset or remove this device from your system before continuing."
            )

        device = device.force_upgrade()
        output.append(device)

    return output



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0

from rich.style import Style


def create_color_scheme(color_system: str):
    if color_system == "truecolor":
        return {
            "red": "#f04f5e",
            "green": "#1eb57d",
            "yellow": "#ffd10a",
            "purple": "#786bb0",
            "orange": "dark_orange",
            "white": "white",
            "grey": "bright_black",
        }
    elif color_system == "256":
        return {
            "red": "red1",
            "green": "dark_cyan",
            "yellow": "gold1",
            "purple": "medium_purple3",
            "orange": "dark_orange",
            "white": "white",
            "grey": "bright_black",
        }
    elif color_system == "standard":
        return {
            "red": "red",
            "green": "green",
            "yellow": "bright_yellow",
            "purple": "magenta",
            "orange": "bright_red",
            "white": "white",
            "grey": "bright_black",
        }


class CMD_LINE_COLOR:
    PURPLE = "\033[95m"
    BLUE = "\033[94m"
    GREEN = "\033[92m"
    RED = "\033[91m"
    ENDC = "\033[0m"
    YELLOW = "\033[93m"


def create_tt_tools_theme():
    """Return tt-health theme"""
    tt_colors = create_color_scheme("truecolor")

    return {
        "yellow_bold": Style(color=tt_colors["yellow"], bold=True),
        "orange_italic": Style(color=tt_colors["orange"], italic=True),
        "yellow_italic": Style(color=tt_colors["yellow"], italic=True),
        "light_green_bold": Style(color="#B0FC38", bold=True),
        "red_bold": Style(color=tt_colors["red"], bold=True),
        "gray": Style(color=tt_colors["grey"]),
        "text_green": Style(color=tt_colors["green"]),
        "warning": Style(color=tt_colors["red"]),
        "attention": Style(color=tt_colors["orange"]),
        "green_bold": Style(color=tt_colors["green"], bold=True),
    }



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0

"""
This file contains functions used to generate and save reset logs
"""

import os
import sys
import json
import datetime
from pathlib import Path
from typing import Dict, Union
from enum import Enum
from dataclasses import dataclass
import tt_tools_common.reset_common.host_reset_log as log
from tt_tools_common.ui_common.themes import CMD_LINE_COLOR
from tt_tools_common.utils_common.system_utils import get_host_info
from tt_tools_common.utils_common.tools_utils import init_logging

LOG_FOLDER = os.path.expanduser("~/.config/tenstorrent")

class ResetType(Enum):
    ALL = 1
    CONFIG_JSON = 2
    ID_LIST = 3

@dataclass
class ResetInput:
    type: ResetType
    value: Union[Dict, int, None]

def parse_reset_input(value: list) -> ResetInput:
    """
    Attempt to parse a reset argument as one of three ResetTypes:

    - String literal "all" or no input (ALL)
    - JSON file (CONFIG_JSON)
    - List of ints corresponding to PCIe indices (ID_LIST)
        - Note it is valid for these ints to be comma or space separated

    Returns a ResetInput with a type and value.
    """
    if value == [] or value == ["all"]:
        return ResetInput(type = ResetType.ALL, value = None)

    if len(value) == 1: # No spaces in input- could be a filename or "0,1,2"-like
        str_input = value[0]
        try:
            # Attempt to parse as a JSON file
            with open(str_input, "r") as json_file:
                json_data: dict = json.load(json_file)
                return ResetInput(type = ResetType.CONFIG_JSON, value = json_data)

        except json.JSONDecodeError as e:
            print(
                CMD_LINE_COLOR.RED,
                f"Please check the format of the json file.\n {e}",
                CMD_LINE_COLOR.ENDC,
            )

        except FileNotFoundError:
            # If no file found, attempt to parse the string as a list of comma separated integers
            try:
                list_input = [int(item) for item in str_input.split(",")]
                list_input = list(sorted(set(list_input))) # Filter repeats
                return ResetInput(type = ResetType.ID_LIST, value = list_input)
            except ValueError:
                print(
                    CMD_LINE_COLOR.RED,
                    "Invalid input! Provide list of comma separated numbers or a json file.\n To generate a reset json config file run tt-smi -g",
                    CMD_LINE_COLOR.ENDC,
                )
                sys.exit(1)

    else: # Spaces in input- should be a list of ints
        try:
            list_input = [int(item) for item in value]
            list_input = list(sorted(set(list_input))) # Filter repeats
            return ResetInput(type = ResetType.ID_LIST, value = list_input)
        except ValueError as e:
                print(
                    CMD_LINE_COLOR.RED,
                    "Invalid input! Provide list of comma separated numbers or a json file.\n To generate a reset json config file run tt-smi -g",
                    CMD_LINE_COLOR.ENDC,
                )
                sys.exit(1)

def generate_reset_logs(devices, result_filename: str = None):
    """
    Generate and save reset logs
    Separate PCI indexes for gs and wh devices
    Generate a reset log with a sample mobo reset config
    """

    time_now = datetime.datetime.now()
    gs_pci_idx = []
    wh_pci_idx = []
    for i, dev in enumerate(devices):
        if dev.as_wh() and not dev.is_remote():
            wh_pci_idx.append(dev.get_pci_interface_id())
        elif dev.as_gs():
            gs_pci_idx.append(dev.get_pci_interface_id())
    reset_log = log.HostResetLog(
        time=time_now,
        host_name=get_host_info()["Hostname"],
        gs_tensix_reset=log.PciResetDeviceInfo(pci_index=gs_pci_idx),
        wh_link_reset=log.PciResetDeviceInfo(pci_index=wh_pci_idx),
        re_init_devices=True,
        disable_serial_report=False,
        disable_sw_version_report=False,
        wh_mobo_reset=[
            log.MoboReset(
                nb_host_pci_idx=wh_pci_idx,
                mobo="<MOBO NAME>",
                credo=["<group id>:<credo id>", "<group id>:<credo id>"],
                disabled_ports=["<group id>:<credo id>", "<group id>:<credo id>"],
            ),
            log.MoboReset(
                nb_host_pci_idx=wh_pci_idx,
                mobo="<MOBO NAME>",
                credo=["<group id>:<credo id>", "<group id>:<credo id>"],
                disabled_ports=["<group id>:<credo id>", "<group id>:<credo id>"],
            ),
        ],
    )
    if result_filename:
        dir_path = os.path.dirname(os.path.realpath(result_filename))
        Path(dir_path).mkdir(parents=True, exist_ok=True)
        log_filename = result_filename
    else:
        log_filename = f"{LOG_FOLDER}/reset_config.json"
        if not os.path.exists(LOG_FOLDER):
            init_logging(LOG_FOLDER)
    reset_log.save_as_json(log_filename)
    return log_filename



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0
"""
Test for generating reset config file
"""
import json
from pyluwen import detect_chips
from tt_tools_common.reset_common.reset_utils import (
    generate_reset_logs,
    parse_reset_input,
)
from tt_tools_common.ui_common.themes import CMD_LINE_COLOR


def main():
    devices = detect_chips()
    file = generate_reset_logs(devices)
    print(
        CMD_LINE_COLOR.BLUE,
        f"Generated default reset config file for this host: {file}",
        CMD_LINE_COLOR.ENDC,
    )

    file = generate_reset_logs(devices, "test_reset_config.json")
    print(
        CMD_LINE_COLOR.BLUE,
        f"Generated input reset config file for this host: {file}",
        CMD_LINE_COLOR.ENDC,
    )
    print(
        CMD_LINE_COLOR.PURPLE,
        f"Parsed output: {file}",
        CMD_LINE_COLOR.ENDC,
    )
    json_data = parse_reset_input(file)
    json_formatted_str = json.dumps(json_data, indent=2)
    print(json_formatted_str)
    nb_host_pci_idx_list = []
    for entry in json_data["wh_mobo_reset"]:
        if "nb_host_pci_idx" in entry.keys() and entry["nb_host_pci_idx"]:
            nb_host_pci_idx_list.extend(entry["nb_host_pci_idx"])
    print("nb_host_pci_idx_list: ", list(set(nb_host_pci_idx_list)))
    pci_idx = parse_reset_input("0,1,2,3")
    print(
        CMD_LINE_COLOR.PURPLE,
        f"Example pci index input: {file}",
        CMD_LINE_COLOR.ENDC,
    )
    print(pci_idx)


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0

"""
This file contains functions used to reset Tensix cores on a Grayskull chip.
"""

import os
import functools
import itertools
from pyluwen import PciChip
from tt_tools_common.utils_common.registers import get_register_object
from tt_tools_common.ui_common.themes import CMD_LINE_COLOR
from tt_tools_common.utils_common.tools_utils import init_fw_defines, int_to_bits


class GSTensixReset:
    """
    This class is used to reset Tensix cores on a GS chip.
    """

    # GS Magic numbers for Tensix resetting and NOC setup
    NIU_CFG_0 = 0x100  # bit 0 is CG enable, bit 12 is tile clock disable
    ROUTER_CFG_0 = 0x104  # bit 0 is CG enable

    ROUTER_CFG_1 = 0x108
    ROUTER_CFG_3 = 0x110
    GRID_SIZE_X = 13
    GRID_SIZE_Y = 12
    NUM_TENSIX_X = GRID_SIZE_X - 1
    NUM_TENSIX_Y = GRID_SIZE_Y - 2
    NOC_MAX_BACKOFF_EXP = 0xF
    PHYS_Y_TO_NOC_0_Y = [0, 11, 1, 10, 2, 9, 3, 8, 4, 7, 5, 6]
    DRAM_LOCATIONS = [
        (1, 6),
        (4, 6),
        (7, 6),
        (10, 6),
        (1, 0),
        (4, 0),
        (7, 0),
        (10, 0),
    ]
    ARC_LOCATIONS = [(0, 2)]
    PCI_LOCATIONS = [(0, 4)]

    def __init__(
        self,
        device: PciChip,
    ):
        self.ROUTING_TO_PHYSICAL_TABLE = dict()
        for phys_y in range(self.GRID_SIZE_Y):
            for phys_x in range(self.GRID_SIZE_X):
                rout_x = self._physical_to_routing(self.GRID_SIZE_X, phys_x)
                rout_y = self._physical_to_routing(self.GRID_SIZE_Y, phys_y)
                self.ROUTING_TO_PHYSICAL_TABLE[(rout_x, rout_y)] = (phys_x, phys_y)
        self.device = device
        self.axi_registers = get_register_object(device)
        self.fw_defines = init_fw_defines("grayskull", "tt_tools_common.data")
        self.harvested_rows = self.get_harvested_rows()
        self.core_list = self.get_core_list()

    @staticmethod
    def _physical_to_routing(limit, phys):
        if (phys % 2) == 0:
            rout = phys // 2
        else:
            rout = limit - 1 - (phys // 2)
        return rout

    def get_harvested_rows(self):
        """
        Get the rows that are harvested on the chip based on efuses.
        """
        if "T6PY_HARVESTING_OVERRIDE" in os.environ:
            harvesting_fuses = int(os.environ["T6PY_HARVESTING_OVERRIDE"], 0)
        else:
            harvesting_fuses, exit_code = self.device.arc_msg(
                self.fw_defines["MSG_TYPE_ARC_GET_HARVESTING"],
                wait_for_done=True,
                arg0=0,
                arg1=0,
            )
            if exit_code != 0:
                assert False, "FW is too old, please update fw"
        bad_mem_bits = harvesting_fuses & 0x3FF
        bad_logic_bits = (harvesting_fuses >> 10) & 0x3FF

        bad_row_bits = (bad_mem_bits | bad_logic_bits) << 1
        bad_physical_rows = int_to_bits(bad_row_bits)
        disabled_rows = frozenset(
            map(
                lambda y: self.PHYS_Y_TO_NOC_0_Y[self.GRID_SIZE_Y - y - 1],
                bad_physical_rows,
            )
        )
        return disabled_rows

    def get_core_list(self):
        """
        Using the harvesting fuses, get a list of Tensix cores that are not harvested.

        Returns:
            Dict of cores that are not harvested
        """
        good_rows = filter(
            lambda y: y not in self.harvested_rows, [1, 2, 3, 4, 5, 7, 8, 9, 10, 11]
        )
        good_cores = list(
            itertools.product(list(range(1, self.GRID_SIZE_X)), good_rows)
        )
        core_list = {
            c: None for c in good_cores
        }  # dict for deterministic order and fast contains checks.
        return core_list

    def get_noc_router_cfg(self):
        """
        Compute the noc router settings for broadcast disable based on harvested rows.
        """
        broadcast_disabled_rows = [0, 6]
        broadcast_disabled_rows += self.harvested_rows
        noc0_router_cfg_1 = 1 << 0  # disable broadcast to column 0
        noc1_router_cfg_1 = (
            1 << 12
        )  # remap noc0 to noc1: disable broadcast to column 12
        noc0_router_cfg_3 = functools.reduce(
            int.__or__, (1 << y for y in broadcast_disabled_rows), 0
        )
        broadcast_disabled_rows_noc1 = map(
            lambda y: self.GRID_SIZE_Y - y - 1, broadcast_disabled_rows
        )
        noc1_router_cfg_3 = functools.reduce(
            int.__or__, map(lambda y: 1 << y, broadcast_disabled_rows_noc1), 0
        )
        return (
            noc0_router_cfg_1,
            noc1_router_cfg_1,
            noc0_router_cfg_3,
            noc1_router_cfg_3,
        )

    def set_safe_clks(self, enter_safe_clks: bool):
        """
        Send arc msg to enter safe clks mode.
        It lowers the clks to a safe level to toggle Tensix resets
        """
        if enter_safe_clks:
            self.device.arc_msg(
                self.fw_defines["MSG_TYPE_RESET_SAFE_CLKS"],
                wait_for_done=True,
                arg0=1,
                arg1=0,
            )
        else:
            # Return clks to before safe clks mode
            self.device.arc_msg(
                self.fw_defines["MSG_TYPE_RESET_SAFE_CLKS"],
                wait_for_done=True,
                arg0=0,
                arg1=0,
            )

    def all_riscs_assert_hard_reset(self):
        """
        Place all risc's into hard reset. This requires set_safe_clks to be run prior.
        """
        for i in range(8):
            self.axi_registers.write32(f"ARC_RESET.RISCV_RESET[{i}]", 0x0)

    def tensix_toggle_reset(self):
        """
        Put all Tensix cores into hard reset and takes them out again.
        This requires set_safe_clks to be run prior.
        This includes both harvested and non-harvested Tensix cores because we need all Tensix cores' NOC routers.
        """
        for i in range(8):
            self.axi_registers.write32(f"ARC_RESET.TENSIX_RESET[{i}]", 0x0)
        for i in range(8):
            self.axi_registers.write32(f"ARC_RESET.TENSIX_RESET[{i}]", 0xFFFFFFFF)

    def is_tensix_core_loc(self, x, y):
        """
        Checks if the given noc 0 coordinate is a Tensix core.
        """
        return (x, y) in self.core_list

    def setup_noc(self):
        """
        This configures all NOC notes.
        It sets broadcast disables, CG enables, and maximum exponential backoff.
        It activates the tile clk disable for harvested Tensix cores.
        """
        self.axi_registers.write_fields(
            "ARC_RESET.DDR_RESET", {"axi_reset": 1, "ddrc_reset": 1}, init=0
        )
        device = self.device.as_gs()
        (
            noc0_router_cfg_1,
            noc1_router_cfg_1,
            noc0_router_cfg_3,
            noc1_router_cfg_3,
        ) = self.get_noc_router_cfg()

        # Write all the registers for NOC NIU & router on a node.
        # x, y are NOC0 coordinates. reg_bases[i] is NOCi reg base address
        def setup_noc_by_xy(x, y, reg_bases):
            # x, y are NOC-noc_id coordinates. noc_reg_base is for noc_id.
            # This writes values that are NOC-independent.
            def setup_noc_common(x, y, noc_id, noc_reg_base):
                # CG enable
                rmw_val = device.noc_read32(noc_id, x, y, noc_reg_base + self.NIU_CFG_0)
                device.noc_write32(
                    noc_id, x, y, noc_reg_base + self.NIU_CFG_0, rmw_val | 0x1
                )
                rmw_val = device.noc_read32(
                    noc_id, x, y, noc_reg_base + self.ROUTER_CFG_0
                )
                device.noc_write32(
                    noc_id, x, y, noc_reg_base + self.ROUTER_CFG_0, rmw_val | 0x1
                )

                # maximum exponential backoff
                rmw_val = device.noc_read32(
                    noc_id, x, y, noc_reg_base + self.ROUTER_CFG_0
                )
                rmw_val &= ~0xF00
                rmw_val |= self.NOC_MAX_BACKOFF_EXP << 8
                device.noc_write32(
                    noc_id, x, y, noc_reg_base + self.ROUTER_CFG_0, rmw_val
                )

            n1x = self.GRID_SIZE_X - x - 1
            n1y = self.GRID_SIZE_Y - y - 1

            device.noc_write32(
                0, x, y, reg_bases[0] + self.ROUTER_CFG_1, noc0_router_cfg_1
            )
            device.noc_write32(
                0, x, y, reg_bases[0] + self.ROUTER_CFG_3, noc0_router_cfg_3
            )
            device.noc_write32(
                1, n1x, n1y, reg_bases[1] + self.ROUTER_CFG_1, noc1_router_cfg_1
            )
            device.noc_write32(
                1, n1x, n1y, reg_bases[1] + self.ROUTER_CFG_3, noc1_router_cfg_3
            )

            setup_noc_common(x, y, 0, reg_bases[0])
            setup_noc_common(n1x, n1y, 1, reg_bases[1])

        # Write all the registers for NOC NIU & router on a node.
        # reg_bases[i] are PCI BAR-relative addresses for registers.
        def setup_noc_by_address(reg_bases):
            # This writes values that are NOC-independent.
            def setup_noc_common(noc_reg_base):
                # CG enable
                rmw_val = device.pci_axi_read32(noc_reg_base + self.NIU_CFG_0)
                # print(f"rmw_val NIU_CFG_0 = {rmw_val}")
                device.pci_axi_write32(noc_reg_base + self.NIU_CFG_0, rmw_val | 0x1)
                rmw_val = device.pci_axi_read32(noc_reg_base + self.ROUTER_CFG_0)
                # print(f"rmw_val ROUTER_CFG_0 = {rmw_val}")
                device.pci_axi_write32(noc_reg_base + self.ROUTER_CFG_0, rmw_val | 0x1)

                # maximum exponential backoff
                rmw_val = device.pci_axi_read32(noc_reg_base + self.ROUTER_CFG_0)
                rmw_val &= ~0xF00
                rmw_val |= self.NOC_MAX_BACKOFF_EXP << 8
                # device.pci_axi_write32(noc_reg_base + ROUTER_CFG_0, rmw_val)

            device.pci_axi_write32(reg_bases[0] + self.ROUTER_CFG_1, noc0_router_cfg_1)
            device.pci_axi_write32(reg_bases[0] + self.ROUTER_CFG_3, noc0_router_cfg_3)
            device.pci_axi_write32(reg_bases[1] + self.ROUTER_CFG_1, noc1_router_cfg_1)
            device.pci_axi_write32(reg_bases[1] + self.ROUTER_CFG_3, noc1_router_cfg_3)

            setup_noc_common(reg_bases[0])
            setup_noc_common(reg_bases[1])

        for y in range(0, self.GRID_SIZE_Y):
            for x in range(0, self.GRID_SIZE_X):
                if (x, y) in self.DRAM_LOCATIONS:
                    setup_noc_by_xy(x, y, [0xFFFF4000, 0xFFFF5000])

                elif (x, y) in self.ARC_LOCATIONS:
                    setup_noc_by_address([0x1FF50000, 0x1FF58000])

                elif (x, y) in self.PCI_LOCATIONS:
                    setup_noc_by_address([0x1FD00000, 0x1FD08000])

                elif x == 0 or y == 0 or y == 6:
                    setup_noc_by_xy(x, y, [0xFFB20000, 0xFFB30000])

                else:
                    # Tensix core node
                    noc_reg_bases = [0xFFB20000, 0xFFB30000]
                    setup_noc_by_xy(x, y, noc_reg_bases)

                    # Set NIU_CFG_0 tile clock disable based on core harvesting.
                    rmw_val = device.noc_read32(
                        0, x, y, noc_reg_bases[0] + self.NIU_CFG_0
                    )
                    rmw_val &= ~(1 << 12)
                    rmw_val |= (0 if self.is_tensix_core_loc(x, y) else 1) << 12
                    device.noc_write32(
                        0, x, y, noc_reg_bases[0] + self.NIU_CFG_0, rmw_val
                    )

                    n1x = self.GRID_SIZE_X - x - 1
                    n1y = self.GRID_SIZE_Y - y - 1

                    rmw_val = device.noc_read32(
                        1, n1x, n1y, noc_reg_bases[1] + self.NIU_CFG_0
                    )
                    rmw_val &= ~(1 << 12)
                    rmw_val |= (0 if self.is_tensix_core_loc(x, y) else 1) << 12
                    device.noc_write32(
                        1, n1x, n1y, noc_reg_bases[1] + self.NIU_CFG_0, rmw_val
                    )

    def assert_all_riscv_soft_reset(self):
        """
        Assert all riscv soft resets using a noc broadcast.
        This reset includes only un-harvested Tensix cores.
        """
        device = self.device.as_gs()
        BRISC_SOFT_RESET = 1 << 11
        TRISC_SOFT_RESETS = (1 << 12) | (1 << 13) | (1 << 14)
        NCRISC_SOFT_RESET = 1 << 18
        device.noc_broadcast32(
            0, 0xFFB121B0, BRISC_SOFT_RESET | TRISC_SOFT_RESETS | NCRISC_SOFT_RESET
        )

    def noc_loc_to_reset_mask(self, noc_x, noc_y):
        """
        Get the reset bit location for a specific Tensix core.
        This applies to both RISCV_RESET and TENSIX_RESET registers.
        """
        phys_x, phys_y = self.ROUTING_TO_PHYSICAL_TABLE[(noc_x, noc_y)]
        blue_x, blue_y = phys_x - 1, phys_y - 1
        reset_bit_index = blue_x * self.NUM_TENSIX_X + blue_y
        reset_reg_index = reset_bit_index // 32
        reset_bit_mask = 1 << (reset_bit_index % 32)
        return reset_reg_index, reset_bit_mask

    def all_tensix_reset_mask(self):
        """
        Get the reset mask for un-harvested Tensix cores.
        """
        reset_mask = [0] * 8

        for x, y in self.core_list.keys():
            index, bit_mask = self.noc_loc_to_reset_mask(x, y)
            reset_mask[index] |= bit_mask

        return reset_mask

    def all_riscs_deassert_hard_reset(self):
        """
        Deassert riscv hard resets. We use a mask to get the un-harvested Tensix cores.
        We only deassert this on un-harvested Tensix cores because un-harvested Tensix cores received the soft reset signal
        The harvested Tensix cores need to stay in a state of hard reset.
        """
        for index, mask in enumerate(self.all_tensix_reset_mask()):
            self.axi_registers.write32(f"ARC_RESET.RISCV_RESET[{index}]", mask)

    def tensix_reset(self, silent=False) -> None:
        """
        This resets all Tensix cores on a GS chip, leaving the un-harvested risc's in soft reset.
        """
        if not silent:
            print(
                CMD_LINE_COLOR.YELLOW,
                "Lowering clks to safe value...",
                CMD_LINE_COLOR.ENDC,
            )
        self.set_safe_clks(True)
        try:
            if not silent:
                print(
                    CMD_LINE_COLOR.YELLOW,
                    "Beginning reset sequence...",
                    CMD_LINE_COLOR.ENDC,
                )
            self.all_riscs_assert_hard_reset()
            self.tensix_toggle_reset()
            self.setup_noc()
            self.assert_all_riscv_soft_reset()
            self.all_riscs_deassert_hard_reset()
            if not silent:
                print(
                    CMD_LINE_COLOR.YELLOW,
                    "Finishing reset sequence...",
                    CMD_LINE_COLOR.ENDC,
                )
        finally:
            if not silent:
                print(
                    CMD_LINE_COLOR.YELLOW,
                    "Returning clks to original values...",
                    CMD_LINE_COLOR.ENDC,
                )
            self.set_safe_clks(False)



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0

"""
This file contains functions used to do a galaxy reset.
"""

import sys
import threading
import requests
import time
from typing import Optional
from tqdm import tqdm

from tt_tools_common.utils_common.system_utils import check_driver_version
from tt_tools_common.reset_common.wh_reset import WHChipReset
from tt_tools_common.ui_common.themes import CMD_LINE_COLOR


class GalaxyReset:
    """Class to perform galaxy reset operations"""

    def threaded_mobo_reset(self, mobo_dict_list, function, args=()):
        """Threaded function to perform mobo reset operations concurrently"""

        class ThreadWrapper(threading.Thread):
            """
            Wrapper class to allow exceptions to be raised from threads,
            needed because exceptions raised in threads will simply cause
            the thread to exit and not stop the program
            """

            def __init__(self, target, args):
                threading.Thread.__init__(self)
                self.target = target
                self.args = args
                self.exc = None

            def run(self):
                try:
                    self.target(*self.args)
                except Exception as e:
                    self.exc = e

        # Thread the function on all mobos
        all_threads = []
        for mobo_dict in mobo_dict_list:
            t = ThreadWrapper(target=function, args=(mobo_dict,) + args)
            all_threads.append(t)
            t.start()

        for t in all_threads:
            t.join()

        # If any exceptions were raised, print them and exit
        exceptions = [t.exc for t in all_threads if t.exc is not None]
        if exceptions:
            for e in exceptions:
                print(e)
            sys.exit(1)

    def mobo_address_generator(self, mobo: str, command: str):
        """Generate url and auth for mobo given a command"""
        url = f"http://{mobo}:8000/{command}"
        auth = ("admin", "admin")
        return url, auth

    def get_server_version(self, mobo):
        # Try to get the server version, but some servers may not have the /about endpoint so defaulting to 0.0.0
        response_url, response_auth = self.mobo_address_generator(mobo, "about")
        try:
            response = requests.get(response_url, auth=response_auth, timeout=30)
            response.raise_for_status()

            response = response.json()
            server_version = tuple(map(int, response["version"].split(".")))
        except Exception:
            server_version = (0, 0, 0)

        return server_version

    def server_communication(
        self,
        post: bool,
        mobo: str,
        command: str,
        data: Optional[dict] = None,
        check_error: bool = True,
    ):
        """Function to communicate with the server and handle errors and exceptions"""
        response_url, response_auth = self.mobo_address_generator(mobo, command)
        if post:
            response = requests.post(response_url, auth=response_auth, json=data)
        else:
            response = requests.get(
                response_url,
                auth=response_auth,
            )

        try:
            exception = None
            response_json = {}  # Initializing just in case...

            # No response for successful shutdown/modules and boot/modules
            if response.text != "":
                response_json = response.json()

            if (
                "error" in response_json
            ):  # Error for boot, shutdown/modules, boot/modules
                exception = f"{mobo} request {command} returned with error {response_json['error']}"
            elif (
                "exception" in response_json and response_json["exception"] is not None
            ):  # Exception for boot/progress
                exception = f"{mobo} request {command} returned with exception {response_json['exception']}"
        except requests.exceptions.HTTPError as e:
            exception = f"{mobo} request {command} failed with HTTP error {e}, response {response.text}"
        except Exception as e:
            # This is somewhat of an unexpected error, so we want to raise it immediately
            raise Exception(
                f"{mobo} request {command} failed with exception {response.text}"
            )
        finally:
            if check_error and exception is not None:
                raise Exception(f"{CMD_LINE_COLOR.RED}{exception}{CMD_LINE_COLOR.ENDC}")

        return response_json

    def credo_boot(self, mobo_dict):
        # Function for booting credos concurrently
        mobo = mobo_dict["mobo"]
        if "credo" in mobo_dict.keys():
            credo_ports = mobo_dict["credo"]
        else:
            print(
                CMD_LINE_COLOR.BLUE,
                f"{mobo} - No credos to be booted, moving on ...",
                CMD_LINE_COLOR.ENDC,
            )
            return
        if "disabled_ports" in mobo_dict.keys():
            disable_ports = mobo_dict["disabled_ports"]
        else:
            disable_ports = []

        server_version = self.get_server_version(mobo)

        # disable_ports is really only a feature for 1.3.2 and above, so if the server version is less than that, then
        # output a message and ignore the disable_ports flag
        boot_data = {"groups": None, "credo": True, "retimer_sel": credo_ports}
        if server_version >= (1, 3, 2):
            boot_data["disable_sel"] = disable_ports
        elif server_version < (1, 3, 2) and disable_ports is not None:
            print(
                f"{CMD_LINE_COLOR.RED}Warning: port disable is only available for server version 1.3.2 and above, ignoring flag for {mobo}{CMD_LINE_COLOR.ENDC}"
            )

        print(CMD_LINE_COLOR.BLUE, f"{mobo} - Booting credo ...", CMD_LINE_COLOR.ENDC)
        self.server_communication(True, mobo, "boot", boot_data)

    def wait_for_boot_complete(self, mobo_dict, mobo_list, timeout=600):
        """Function to wait for boot completion of the server and update progress bar accordingly"""
        mobo = mobo_dict["mobo"]
        # Do a check for the server version, if it is >= 0.3.0, then the boot command is posted and
        # we need to do a while loop to check for boot progress
        server_version = self.get_server_version(mobo)
        if server_version < (0, 3, 0):
            return

        # Derive position from mobo_list
        position = mobo_list.index(mobo)
        progress_bar = tqdm(
            total=100, bar_format="{desc} [{elapsed}]", position=position
        )

        # Get the update percentage and update the progress bar
        time_start = time.time()
        boot_progress = 0.0
        progress_bar.set_description_str(
            f"{mobo} - Waiting for server boot to complete... {boot_progress:6.2f}%"
        )
        while boot_progress < 100.0:
            if time.time() - time_start > timeout:
                raise Exception(
                    f"{CMD_LINE_COLOR.RED}{mobo} - Boot timeout, please power cycle the galaxy and try boot again{CMD_LINE_COLOR.ENDC}"
                )

            response = self.server_communication(False, mobo, "boot/progress")

            boot_progress = float(response["boot_percent"])

            # If the server version is >= 1.3.2, then we have a more verbose display of boot status
            if server_version >= (1, 3, 2):
                extra_info = f" ({response['step']})"
            else:
                extra_info = ""

            progress_bar.set_description_str(
                f"{mobo} - Waiting for server boot to complete... {boot_progress:6.2f}%{extra_info}"
            )
            time.sleep(1)

    def shutdown_modules(self, mobo_dict):
        # Function for shutting down modules concurrently

        mobo = mobo_dict["mobo"]
        print(
            CMD_LINE_COLOR.PURPLE,
            f"{mobo} - Turning off modules ...",
            CMD_LINE_COLOR.ENDC,
        )
        self.server_communication(
            True, mobo, "shutdown/modules", {"groups": None}, check_error=False
        )

    def boot_modules(self, mobo_dict):
        # Function for booting modules concurrently
        mobo = mobo_dict["mobo"]
        print(
            CMD_LINE_COLOR.PURPLE,
            f"{mobo} - Turning on modules ...",
            CMD_LINE_COLOR.ENDC,
        )
        self.server_communication(True, mobo, "boot/modules", {"groups": None})

    def warm_reset_mobo(self, mobo_dict_list):
        """Warm boot mobos in dict list form json file"""
        check_driver_version("boot mobo")

        #  Credo boot if needed
        self.threaded_mobo_reset(mobo_dict_list, self.credo_boot)

        # Poll for boot completion status if needed
        mobo_list = [entry["mobo"] for entry in mobo_dict_list]
        self.threaded_mobo_reset(
            mobo_dict_list, self.wait_for_boot_complete, (mobo_list,)
        )

        # Shutdown modules
        self.threaded_mobo_reset(mobo_dict_list, self.shutdown_modules)

        # Link reset the NB host connected to the mobo
        nb_host_pci_idx_list = []
        for entry in mobo_dict_list:
            if "nb_host_pci_idx" in entry.keys() and entry["nb_host_pci_idx"]:
                nb_host_pci_idx_list.extend(entry["nb_host_pci_idx"])
        if nb_host_pci_idx_list:
            # remove duplicate entries
            nb_host_pci_idx_list = list(set(nb_host_pci_idx_list))
            WHChipReset().full_lds_reset(nb_host_pci_idx_list)

        # Boot modules after reset
        self.threaded_mobo_reset(mobo_dict_list, self.boot_modules)

    def mobo_reset_from_json(self, json_dict):
        """Parse pci_list from reset json and init mobo reset"""
        if "wh_mobo_reset" in json_dict.keys():
            mobo_dict_list = []
            for mobo_dict in json_dict["wh_mobo_reset"]:
                if "MOBO NAME" not in mobo_dict["mobo"]:
                    mobo_dict_list.append(mobo_dict)
            # If any mobos - do the reset
            if mobo_dict_list:
                self.warm_reset_mobo(mobo_dict_list)



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0

from textual.app import RenderResult, ComposeResult
from textual.widget import Widget
from textual.widgets import Button, DataTable, Label, Switch, Checkbox
from textual.widgets.data_table import CellDoesNotExist
from textual.containers import ScrollableContainer, Container, Grid, Horizontal
from textual.coordinate import Coordinate
from textual.screen import ModalScreen
from textual.widgets import Markdown
from textual import on
from rich.panel import Panel
from rich.table import Table
from rich.style import Style
from rich.text import Text

from datetime import datetime
from typing import OrderedDict, Dict, List, Tuple, Callable, Union


class TTHeader(Widget):
    """A custom header widget for TT-Tools."""

    def __init__(self, app_name: str, app_version: str) -> None:
        super().__init__()
        self.app_name = app_name
        self.app_version = app_version

    def on_mount(self) -> None:
        self.set_interval(1, callback=self.refresh)

    def render(self) -> RenderResult:
        grid = Table.grid(expand=True, padding=(0, 1), pad_edge=True)
        grid.add_column(justify="left", ratio=0.25)
        grid.add_column(justify="center", ratio=0.5)
        grid.add_column(justify="right", ratio=0.25)

        version = f"Version {self.app_version}"
        app_name = self.app_name
        date = datetime.now().strftime("%b %d %Y %I:%M:%S %p")

        grid.add_row(version, app_name, date)

        return Panel(grid)


class TTFooter(Widget):
    """A custom footer widget for TT-Tools."""

    def __init__(self, content: List[str], justify: str = "center") -> None:
        super().__init__()
        self.content = content
        self.justify = justify

    def render(self) -> RenderResult:
        grid = Table.grid(expand=True, collapse_padding=False)
        for _ in range(len(self.content)):
            grid.add_column(justify=self.justify, ratio=1)
        grid.add_row(*self.content)

        return Panel(grid)


class TTDataTable(ScrollableContainer):
    """A custom container with a DataTable for TT-Tools."""

    def __init__(
        self, title: str, header: List[str] = None, id: str = None, **kwargs
    ) -> None:
        super().__init__(id=id)
        self.border_title = title
        self._title = title
        self.header = header
        self.dt = self.config_dt(**kwargs)

    def style_header(self):
        for i, header_text in enumerate(self.header):
            self.header[i] = Text(header_text, justify="center", style="underline")

    def config_dt(self, **kwargs) -> DataTable:
        # initialize DataTable
        dt = DataTable(id=self.id + "_table", **kwargs)

        # DT props
        dt.zebra_stripes = False
        dt.cursor_type = "cell"
        dt.border_title = self._title

        # TODO: if it doesn't make sense to initialize TTDataTable with header, can move it out to app
        # initialize Header
        if self.header:
            self.style_header()
            dt.add_columns(*self.header)

        return dt

    def update_data(self, rows: List[str]) -> None:
        # assert len(self.header) == max(
        #     [len(x)
        #      for x in rows]), "Data doesn't have expected num of columns"

        for i, row in enumerate(rows):
            for j, val in enumerate(row):
                try:
                    # update cell values one by one
                    self.dt.update_cell_at(
                        coordinate=Coordinate(column=j, row=i), value=val
                    )
                except CellDoesNotExist:
                    # there are more rows than there used to be
                    self.dt.clear()
                    self.dt.add_rows(rows)

    def compose(self) -> ComposeResult:
        container = ScrollableContainer(self.dt, id=self.id)
        yield container


class TTMenu(Container):
    """A custom Menu/List widget for TT-Tools."""

    def __init__(self, id: str, title: str, data: OrderedDict[str, str]) -> None:
        super().__init__(id=id)
        self.border_title = title
        self.data = data
        self.justify_width = max([len(k) for k in data.keys()]) + 1

    def render(self) -> RenderResult:
        text = Text()
        for key, value in self.data.items():
            k = Text(
                f"{key.ljust(self.justify_width)}",
                style=Style(color="#ffd10a", bold=True),
            )
            if key == "Failed to fetch":
                text.append_text(k).append_text(
                    Text(f"\n{value}\n", style=Style(color="dark_orange"))
                )
            else:
                text.append_text(Text("* ")).append_text(k).append_text(
                    Text(f": {value}\n")
                )
        text.rstrip()

        return text


class TTHostCompatibilityMenu(Container):
    """
    A custom Menu/List widget for TT-Tools.
    Used in tt-smi to render Host Info with
    compatibility notes.

    Accepts dicts with string keys. If items
    are str, renders each item to a line akin
    to TTMenu. If item is Tuple of str, renders
    first item to the first line and second item
    to another line in red as a note.
    """

    def __init__(self, id: str, title: str, data: Dict[str, Union[str, Tuple]]) -> None:
        super().__init__(id=id)
        self.data = data
        self.justify_width = max([len(k) for k in self.data.keys()]) + 1

        # Are all values in the dict strings?
        fully_compatible = all([type(value) is str for value in self.data.values()])
        if fully_compatible:
            self.border_title = title + " (Fully Compatible)"
        else:
            self.border_title = title + " (Config Warning!)"
            self.styles.border_title_color = "red"

            self.border_subtitle = "* Recommended Config"
            self.styles.border_subtitle_color = "red"

    def render(self) -> RenderResult:
        text = Text()
        # Track the number of lines to set the widget height
        num_lines = 0
        for key, value in self.data.items():
            line_leader = Text("* ")
            # Render only one line
            if type(value) is str:
                num_lines += 1
                k = Text(
                    f"{key.ljust(self.justify_width)}" + ": ",
                    style=Style(color="#ffd10a", bold=True),
                )
                v = Text(f"{value}\n")
                text.append_text(line_leader).append_text(k).append_text(v)
            # Render two lines, the current state and the recommendation
            elif type(value) is tuple:
                num_lines += 2
                k = Text(
                    f"{key.ljust(self.justify_width)}" + ": ",
                    style=Style(color="#ffd10a", bold=True),
                )
                v_1 = Text(f"{value[0]}\n")
                v_2 = Text(
                    f"{' ' * (self.justify_width)}  * {value[1]}\n",
                    style=Style(color="dark_orange"),
                )

                text.append_text(line_leader).append_text(k).append_text(
                    v_1
                ).append_text(v_2)
            else:
                # TODO: Raise warning here?
                pass
        text.rstrip()

        self.styles.height = (
            4 + num_lines
        )  # Dynamically style widget height to fit all recommendations (+4 for border/padding)

        return text


class TTCompatibilityMenu(Container):
    """A custom Menu/List widget for TT-Tools."""

    def __init__(self, id: str, title: str, data: OrderedDict()) -> None:
        super().__init__(id=id)
        self.border_title = title
        self.data = data
        self.justify_width = max([len(k) for k in data.keys()]) + 1

    def render(self) -> RenderResult:
        text = Text()
        for key, value in self.data.items():
            if value[0] == True:
                k = Text(
                    f"{key.ljust(self.justify_width)}" + ": ",
                    style=Style(color="#ffd10a", bold=True),
                )
                text.append_text(Text("* ")).append_text(k).append_text(
                    Text(f"{value[1]}\n")
                )
            else:
                k = Text(
                    f"{key.ljust(self.justify_width)}" + ": ",
                    style=Style(color="#ffd10a", bold=True),
                )
                text.append_text(Text("* ")).append_text(k).append_text(
                    Text(f"{value[1]}\n", style=Style(color="dark_orange"))
                )
        text.rstrip()

        return text


class TTConfirmBox(ModalScreen):
    """A custom Confirm Box widget for TT-Tools."""

    def __init__(
        self, text: str, on_yes: Callable = None, on_no: Callable = None
    ) -> None:
        super().__init__()
        self.text = text
        self.on_yes = on_yes
        self.on_no = on_no
        # TODO: give border title

    def compose(self) -> ComposeResult:
        yield Grid(
            Label(self.text, id="question"),
            Button("Yes", id="yes"),
            Button("No", id="no"),
            id="dialog",
        )

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "yes" and self.on_yes:
            self.on_yes()
        elif event.button.id == "no" and self.on_no:
            self.on_no()
        self.app.pop_screen()


class TTHelperMenuBox(ModalScreen):
    """A custom modal screen with help options"""

    BINDINGS = [
        ("q", "quit", "Quit"),
        ("escape", "esc_screen", "Escape the open screen"),
        ("h", "help", "Quit the help box"),
    ]

    def __init__(self, text: str, theme: OrderedDict = {}) -> None:
        super().__init__()
        self.text = text
        self.theme = theme

    def compose(self) -> ComposeResult:
        yield Markdown(self.text, id="help_menu_box")

    async def action_quit(self) -> None:
        """Return to main app"""
        self.app.pop_screen()

    async def action_esc_screen(self) -> None:
        """An action to select test to run"""
        self.app.pop_screen()

    async def action_quit(self) -> None:
        """Return to main app"""
        self.app.pop_screen()

    async def action_help(self) -> None:
        """Return to main app"""
        self.app.pop_screen()


class TTSettingsMenu(ModalScreen):
    """A custom modal screen with help options"""

    BINDINGS = [
        ("q, Q, s, S", "quit", "Quit"),
        ("escape", "esc_screen", "Escape the open screen"),
    ]

    def __init__(self, text: str, theme: OrderedDict = {}) -> None:
        super().__init__()
        self.text = text
        self.theme = theme

    def compose(self) -> ComposeResult:
        heading = "TT-SMI Settings"
        instruction = "(tab/mouse/enter: navigate dialogue box & update switches)"
        yield Grid(
            Label(Text(heading, style=self.theme["light_green_bold"]), id="heading"),
            Label(Text(instruction, style=self.theme["attention"]), id="instruction"),
            Label("o Dark Mode:         ", id="dark_label"),
            Switch(value=self.app.dark, id="dark_switch"),
            Label("o Latest SW versions:", id="sw_label"),
            Switch(value=self.app.get_latest_sw_vers, id="sw_switch"),
            id="settings_menu_box",
        )

    @on(Switch.Changed)
    def select_changed(self, event: Switch.Changed) -> None:
        """Switch event handler"""
        if event.switch.id == "dark_switch":
            # Toggle dark mode for app
            if event.switch.value == True:
                self.app.dark = True
            else:
                self.app.dark = False
        if event.switch.id == "sw_switch":
            # Toggle getting latest sw versions
            if event.switch.value == True:
                self.app.get_latest_sw_vers = True
            else:
                self.app.get_latest_sw_vers = False

    async def action_setting(self) -> None:
        """Return to main app"""
        self.app.pop_screen()

    async def action_esc_screen(self) -> None:
        """An action to select test to run"""
        self.app.pop_screen()

    async def action_quit(self) -> None:
        """Return to main app"""
        self.app.pop_screen()

    async def action_help(self) -> None:
        """Return to main app"""
        self.app.pop_screen()



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0

"""
EXAMPLE APP FOR TT-TOOLS
Tests all basic widgets and themes for TT-Tools
"""
from typing import List, Tuple
from textual.app import App, ComposeResult
from textual.containers import Container, Horizontal, Vertical
from textual.widgets import Footer, Header, Static, DataTable
from textual.css.query import NoMatches

from tt_tools_common.ui_common.widgets import (
    TTHeader,
    TTFooter,
    TTDataTable,
    TTMenu,
    TTConfirmBox,
)

from datetime import datetime

TextualKeyBindings = List[Tuple[str, str, str]]


class TTApp(App):
    """A Textual app example to test all tt_textual widgets for TT-Tools."""

    BINDINGS = [
        ("d", "toggle_dark", "Toggle dark mode"),
        ("q", "quit", "Quit"),
        ("b", "test_confirmbox", "Test Confirm Box"),
    ]

    CSS_PATH = "../ui_common/common_style.css"

    def __init__(
        self,
        app_name: str = "TT-App Example",
        app_version: str = "1.0.0",
        key_bindings: TextualKeyBindings = [],
    ) -> None:
        """Initialize the textual app."""
        super().__init__()
        self.app_name = app_name
        self.app_version = app_version

        if key_bindings:
            self.BINDINGS += key_bindings

    def compose(self) -> ComposeResult:
        """Create child widgets for the app."""
        data = {"Menu 1": 1, "Menu 2": 2, "Menu 3": 3}
        yield TTHeader(self.app_name, self.app_version)
        with Horizontal():
            with Vertical():  # TODO: Specify widget width & height on layout
                yield TTMenu(title="Menu 1", data=data, id="menu_1")
                yield TTMenu(title="Menu 2", data=data, id="menu_2")
            yield TTDataTable(
                title="TT_DATA_TABLE_EXAMPLE",
                header=["Board ID", "Test NOC", "Test PCIE", "Time"],
                id="dt_example",
            )
            yield TTMenu(title="Right Menu", data=data, id="menu_3")
        yield Footer()

    def on_mount(self) -> None:
        """Event handler called when widget is added to the app."""
        tt_dt = self.get_widget_by_id(id="dt_example")
        # initial data rows for TTDataTable
        tt_dt.dt.add_rows(
            [
                [
                    f"{n}",
                    f"Status {n}",
                    f"This is description {n}",
                    datetime.now().strftime("%b %d %Y %I:%M:%S %p"),
                ]
                for n in range(42)
            ]
        )

    def action_toggle_dark(self) -> None:
        """An action to toggle dark mode."""
        self.dark = not self.dark

    def action_test_confirmbox(self) -> None:
        """An action to test the confirm box toggle."""
        self.push_screen(TTConfirmBox(text="Are you sure you want to...?"))


if __name__ == "__main__":
    TTApp().run()



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0

"""
This file contains common utilities used by all tt-tools.
"""
import os
import sys
import json
import psutil
import distro
import platform
import requests
from typing import Union, Tuple, Dict
from tt_tools_common.ui_common.themes import CMD_LINE_COLOR

MINIMUM_DRIVER_VERSION_LDS_RESET = "1.26.0"
LOG_FOLDER = os.path.expanduser("~/.config/tenstorrent")

def get_size(size_bytes: int, suffix: str = "B") -> str:
    """
    Scale bytes to its proper format
    e.g:
        1253656 => '1.20MB'
        1253656678 => '1.17GB'
    """
    factor = 1024
    for unit in ["", "K", "M", "G", "T", "P"]:
        if size_bytes < factor:
            return f"{size_bytes:.2f} {unit}{suffix}"
        size_bytes /= factor
    return "N/A"


def get_driver_version() -> Union[str, None]:
    """
    Get the version of the Tenstorrent driver
    """
    try:
        with open("/sys/module/tenstorrent/version", "r", encoding="utf-8") as f:
            driver = f.readline().rstrip()
    except Exception:
        driver = None

    return driver

def _parse_version_string(version_str: str) -> Tuple[int, int, int]:
    """
    Parse a version string into a tuple of (major, minor, patch) integers.
    Handles SemVer-like formats including pre-release identifiers and build metadata,
    e.g., "1.34", "1.34.0", "1.34.1-alpha", "1.2.3+build456", "1.4.0-rc1+build42".

    Pre-release identifiers (e.g., "-alpha") and build metadata (e.g., "+build456")
    are stripped to get the core (major, minor, patch) version for comparison.

    Versions with fewer than three numeric parts (e.g., "1.34") will have patch
    (and minor if applicable for a single "1") defaulted to 0.
    """
    if not version_str:
        raise ValueError("Version string cannot be empty")

    # Strip build metadata (text after first '+') first.
    core_version_str = version_str.split("+")[0]

    # Then, strip pre-release identifier (text after '-').
    main_version_part = core_version_str.split("-")[0]

    parts = main_version_part.split(".")

    if not parts or not parts[0]: # Check for empty string after split or no parts
        raise ValueError(f"Invalid version format: {version_str}")

    try:
        major = int(parts[0])
        minor = int(parts[1]) if len(parts) > 1 else 0
        patch = int(parts[2]) if len(parts) > 2 else 0
    except ValueError as e:
        raise ValueError(f"Version parts must be integers: {version_str}") from e

    return major, minor, patch

def is_driver_version_at_least(current_version: str, minimum_version: str) -> bool:
    """
    Check if the current driver version meets or exceeds the minimum required version.
    """
    if current_version is None:
        raise ValueError("No Tenstorrent driver detected! Please install the driver using tt-kmd: " +
            "https://github.com/tenstorrent/tt-kmd")
    
    current_tuple = _parse_version_string(current_version)
    minimum_tuple = _parse_version_string(minimum_version)

    # Python's tuple comparison works lexicographically, which is suitable for version numbers.
    # (1, 34, 0) is less than (1, 35, 0)
    # (1, 34, 0) is not less than (1, 34, 0)
    # (2, 0, 0) is greater than (1, 35, 0)
    return current_tuple >= minimum_tuple

def check_driver_version(
    operation: str, minimum_required_version_str: str = MINIMUM_DRIVER_VERSION_LDS_RESET
):
    """
    Check if the currently installed Tenstorrent driver version is sufficient
    for the specified operation and exit if not.

    Compares the current driver version against the minimum required version.
    Exits with a non-zero status code if:
    - No driver is detected.
    - Version strings are malformed.
    - The current driver version is older than the minimum required version.
    """
    current_driver_version_str = get_driver_version()

    try:
        if is_driver_version_at_least(current_driver_version_str, minimum_required_version_str):
            return  # Version is sufficient, continue normally
    except ValueError as e:
        print(
            f"{CMD_LINE_COLOR.RED}"
            f"Error parsing driver version. {e}"
            f"{CMD_LINE_COLOR.ENDC}"
        )
        sys.exit(1)

    # If we reach here, the version is insufficient
    print(
        f"{CMD_LINE_COLOR.RED}"
        f"Current driver version: {current_driver_version_str}"
        f"{CMD_LINE_COLOR.ENDC}"
    )
    print(
        f"{CMD_LINE_COLOR.RED}"
        f"Operation '{operation}' requires Tenstorrent driver version "
        f"{minimum_required_version_str} or greater. Your version is too old."
        f"{CMD_LINE_COLOR.ENDC}"
    )
    print(
        f"{CMD_LINE_COLOR.RED}"
        "Please update your driver using tt-kmd: https://github.com/tenstorrent/tt-kmd"
        f"{CMD_LINE_COLOR.ENDC}"
    )
    sys.exit(1)


def get_host_info() -> dict:
    """
        Reads and organizes host info
    Returns:
        dict: with host info
    """
    uname = platform.uname()
    svmem = psutil.virtual_memory()

    os: str = uname.system
    distro_name: str = distro.name(pretty=True)
    kernel: str = uname.release
    hostname: str = uname.node

    return {
        "OS": os,
        "Distro": distro_name,
        "Kernel": kernel,
        "Hostname": hostname,
        "Platform": uname.machine,
        "Python": platform.python_version(),
        "Memory": get_size(svmem.total),
        "Driver": "TT-KMD " + get_driver_version(),
    }

def get_host_compatibility_info() -> Dict[str, Union[str, Tuple]]:
    """
    Return host info with system compatibility notes

    Returns dict with str keys per-item. Values are str
    if the item is fully compatible, or tuple of str if
    not. The first element is the current state and the
    second element is the desired or recommended state.
    """
    host_info = get_host_info()
    checklist = {}
    full_os = f"{host_info['OS']} ({host_info['Platform']})"

    if host_info["OS"] == "Linux":
        checklist["OS"] = full_os
    else:
        checklist["OS"] = (full_os, "Linux (x86_64)")

    if distro.id() == "ubuntu":
        distro_version = float(".".join(distro.version_parts()[:2]))
        if distro_version >= 22:
            checklist["Distro"] = host_info["Distro"]
        else:
            checklist["Distro"] = (host_info["Distro"], "22.04 or 24.04")
    else:
        checklist["Distro"] = (host_info["Distro"], "Ubuntu 22.04 or 24.04")

    checklist["Kernel"] = host_info["Kernel"]
    checklist["Hostname"] = host_info["Hostname"]
    checklist["Python"] = host_info["Python"]

    if psutil.virtual_memory().total >= 32 * 1e9:
        checklist["Memory"] = host_info["Memory"]
    else:
        checklist["Memory"] = (host_info["Memory"], "32GB+")

    if host_info["Driver"]:
        checklist["Driver"] = host_info["Driver"]
    else:
        checklist["Driver"] = (host_info["Driver"], "TT-KMD v1.27+")

    return checklist

def get_sw_ver_flags():
    """
    Get the flags from the reset config file
    """
    if os.path.exists(f"{LOG_FOLDER}/reset_config.json"):
        try:
            # Attempt to parse as a JSON file
            with open(f"{LOG_FOLDER}/reset_config.json", "r") as json_file:
                data = json.load(json_file)
                disable_serial_report = data.get("disable_serial_report", False)
                disable_sw_version_report = data.get("disable_sw_version_report", False)
                return disable_serial_report, disable_sw_version_report
        except json.JSONDecodeError as e:
            return None, None
    else:
        return None, None

def get_sw_ver_from_url(board_id: str):
    url = "https://cereal.tenstorrent.com?SerialNumber=" + board_id
    version = {}
    try:
        r = requests.get(url)

        try:
            r_text = r.json()
        except json.JSONDecodeError:
            print("Error decoding json")
            version["Failed to fetch"] = "No response from server"
        else:
            if isinstance(r_text, dict):
                for key, value in r_text.items():
                    if isinstance(value, str) and isinstance(key, str):
                        version.update({key: value})
                version.update({"Buda": "0.9.80", "Metallium": "0.42.0"})
                # Fix up the keys with user-facing names
                version["TT-Metalium"] = version["Metallium"]
                del version["Metallium"]
                version["TT-Buda"] = version["Buda"]
                del version["Buda"]
            else:
                version["Failed to fetch"] = "Unexpected response from server"
    except requests.exceptions.HTTPError:
        version["Failed to fetch"] = "We encountered an HTTP error."
    except requests.exceptions.ConnectionError:
        version["Failed to fetch"] = (
            "There was an error connecting to the server. Please check your internet connection."
        )
    except requests.exceptions.Timeout:
        version["Failed to fetch"] = (
            "Timeout error. It seems the server is taking too long to respond."
        )
    except requests.exceptions.RequestException:
        version["Failed to fetch"] = "Something unexpected happened."

    return version

def get_sw_ver_info(show_sw_ver: bool, board_ids: str):
    """
    Get software version info.
    If reset_config is available on the system then parse the file and disable settings as specified
    Args:
        show_sw_ver (bool): Whether to show software version info. Will default to N/A if False.
    """
    sw_ver = {
        "Firmware Bundle": "N/A",
        "tt-smi": "N/A",
        "tt-flash": "N/A",
        "tt-kmd": "N/A",
        "TT-Buda": "N/A",
        "TT-Metalium": "N/A",
    }
    disable_serial_report, disable_sw_version_report = get_sw_ver_flags()

    if disable_sw_version_report or not show_sw_ver:
        # return default
        return sw_ver
    if disable_serial_report:
        sw_ver = get_sw_ver_from_url("")
    else:
        for board_id in board_ids:
            # get the query for all the boards
            sw_ver = get_sw_ver_from_url(board_id)
    return sw_ver



# SPDX-FileCopyrightText: © 2023 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0
"""
Test for detect_chips_fallible
"""
from pyluwen import detect_chips
from tt_tools_common.utils_common.tools_utils import detect_chips_with_callback
from tt_tools_common.reset_common.wh_reset import WHChipReset


def main():
    devices = detect_chips_with_callback()

    wh_pci_idx = []
    devices = detect_chips()
    for i, dev in enumerate(devices):
        if dev.as_wh() and not dev.is_remote():
            wh_pci_idx.append(i)

    WHChipReset().full_lds_reset(pci_interfaces=wh_pci_idx)
    devices = detect_chips_with_callback()


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0

"""
This file contains functions used to do a PCIe level reset for Blackhole chip.
"""

import os
import sys
import time
import fcntl
import struct
from typing import List
from pyluwen import PciChip
from tt_tools_common.ui_common.themes import CMD_LINE_COLOR
from tt_tools_common.utils_common.system_utils import (
    get_host_info,
    is_driver_version_at_least,
    get_driver_version
)
from tt_tools_common.reset_common.chip_reset import ChipReset


class BHChipReset:
    """Class to perform a chip level reset on BH PCIe boards"""

    # BH magic numbers for reset
    TENSTORRENT_IOCTL_MAGIC = 0xFA
    TENSTORRENT_IOCTL_RESET_DEVICE = (TENSTORRENT_IOCTL_MAGIC << 8) | 6
    TENSTORRENT_RESET_DEVICE_RESTORE_STATE = 0
    TENSTORRENT_RESET_DEVICE_RESET_PCIE_LINK = 1
    TENSTORRENT_RESET_DEVICE_CONFIG_WRITE = 2
    A3_STATE_PROP_TIME = 0.03
    POST_RESET_MSG_WAIT_TIME = 2
    MSG_TRIGGER_SPI_COPY_LtoR = 0x50
    MSG_TYPE_ARC_STATE3 = 0xA3
    MSG_TYPE_TRIGGER_RESET = 0x56

    def reset_device_ioctl(self, interface_id: int, flags: int) -> bool:
        dev_path = f"/dev/tenstorrent/{interface_id}"
        dev_fd = os.open(
            dev_path, os.O_RDWR | os.O_CLOEXEC
        )  # Raises FileNotFoundError and other appropriate exceptions.
        try:
            reset_device_in_struct = "II"
            reset_device_out_struct = "II"
            reset_device_struct = reset_device_in_struct + reset_device_out_struct

            input_size_bytes = struct.calcsize(reset_device_in_struct)
            output_size_bytes = struct.calcsize(reset_device_out_struct)
            reset_device_buf = bytearray(
                struct.pack(reset_device_struct, output_size_bytes, flags, 0, 0)
            )
            fcntl.ioctl(
                dev_fd, self.TENSTORRENT_IOCTL_RESET_DEVICE, reset_device_buf
            )  # Raises OSError

            output_buf = reset_device_buf[input_size_bytes:]
            _, result = struct.unpack(reset_device_out_struct, output_buf)

            return result == 0
        finally:
            os.close(dev_fd)

    def full_lds_reset(
        self, pci_interfaces: List[int], reset_m3: bool = False, silent: bool = False
    ) -> List[PciChip]:
        """Performs a full LDS reset of a list of chips"""
        
        # Use new reset for driver version >= 2.4.1
        if is_driver_version_at_least(get_driver_version(), "2.4.1"):
            return ChipReset().full_lds_reset(pci_interfaces, reset_m3, silent)

        if not silent:
            print(
                CMD_LINE_COLOR.YELLOW,
                "Notice: Using legacy BH reset implementation. This will be removed in a later version.",
                "Please upgrade tt-kmd to version 2.4.1 or newer to use the updated reset sequence.",
                CMD_LINE_COLOR.ENDC,
            )

        # TODO: FOR BH Check the driver version and bail if link reset cannot be supported
        # check_driver_version(operation="board reset")

        # Due to how Arm systems deal with PCIe device rescans, WH device resets don't work on that platform.
        # Check for platform and bail if it's Arm
        platform = get_host_info()["Platform"]
        if platform.startswith("arm") or platform.startswith("aarch"):
            print(
                CMD_LINE_COLOR.RED,
                "Cannot perform WH board reset on Arm systems, please reboot the system to reset the boards. Exiting...",
                CMD_LINE_COLOR.ENDC,
            )
            sys.exit(1)

        # Remove duplicates from the input list of PCI interfaces
        pci_interfaces = list(set(pci_interfaces))
        if not silent:
            print(
                f"{CMD_LINE_COLOR.BLUE} Starting PCI link reset on BH devices at PCI indices: {str(pci_interfaces)[1:-1]} {CMD_LINE_COLOR.ENDC}"
            )

        pci_bdf_list = {}

        post_reset_wait = self.POST_RESET_MSG_WAIT_TIME

        # Collect device bdf and trigger resets for all BH chips in order
        for pci_interface in pci_interfaces:
            # TODO: Make this check fallible
            chip = PciChip(pci_interface=pci_interface)
            pci_bdf = chip.get_pci_bdf()
            pci_bdf_list[pci_interface] = pci_bdf
            if reset_m3:
                # A full bmfw upgrade can take awhile
                post_reset_wait = 60
                chip.arc_msg(self.MSG_TYPE_TRIGGER_RESET, wait_for_done=False, arg0=3)
            else:
                self.reset_device_ioctl(
                    pci_interface, self.TENSTORRENT_RESET_DEVICE_CONFIG_WRITE
                )

        # check command.memory in config space to see if reset bit is set
        # 0 means config space reset happened correctly
        # 1 means config space reset didn't go through correctly

        completed = 0
        failures = 0
        files_map = {
            pci_interface: open(
                f"/sys/bus/pci/devices/{pci_bdf_list[pci_interface]}/config", "rb"
            )
            for pci_interface in pci_interfaces
        }

        elapsed = 0
        start_time = time.time()
        all_start_time = None
        # Map of pci interface to reset bit
        reset_complete_bit_map = {
            pci_interface: False for pci_interface in pci_interfaces
        }
        can_early_exit = False

        print(
            f"Waiting for up to {post_reset_wait} seconds for asic to come back after reset"
        )
        while elapsed < post_reset_wait:
            for pci_interface, file in files_map.items():
                command_memory_byte = os.pread(file.fileno(), 1, 4)
                reset_bit = (
                    int.from_bytes(command_memory_byte, byteorder="little") >> 1
                ) & 1
                # Overwrite to store the last value
                reset_complete_bit_map[pci_interface] = (
                    True if reset_bit == 0 else False
                )

            # During bmfw upgrade it may take awhile for the asic to go down after sending the message.
            # So to be safe only early exit if we know the asic has actually gone into reset
            if not all(reset_complete_bit_map.values()):
                can_early_exit = True

            if all(reset_complete_bit_map.values()):
                if can_early_exit:
                    break

            time.sleep(0.001)
            elapsed = time.time() - start_time

        # Check the last value of all the reset bits and report if any of them are not 0
        for pci_interface in pci_interfaces:
            if reset_complete_bit_map[pci_interface]:
                print(
                    f"{CMD_LINE_COLOR.GREEN} Config space reset completed for device {pci_interface} {CMD_LINE_COLOR.ENDC}"
                )
                completed += 1
            else:
                print(
                    f"{CMD_LINE_COLOR.RED} Config space reset not completed for device {pci_interface}! {CMD_LINE_COLOR.ENDC}"
                )
                failures += 1

        for pci_interface in pci_interfaces:
            self.reset_device_ioctl(
                pci_interface, self.TENSTORRENT_RESET_DEVICE_RESTORE_STATE
            )

        if failures > 0:
            sys.exit(failures)

        #  All went well print success message
        # other sanity checks go here
        if not silent:
            print(
                f"{CMD_LINE_COLOR.BLUE} Finishing PCI link reset on BH devices at PCI indices: {str(pci_interfaces)[1:-1]} {CMD_LINE_COLOR.ENDC}"
            )

        pci_chips = [PciChip(pci_interface=interface) for interface in pci_interfaces]
        return pci_chips



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0
"""
Reset test for BH Tensix reset
"""
from pyluwen import detect_chips
from tt_tools_common.reset_common.bh_reset import BHChipReset


def main():
    bh_pci_idx = []
    devices = detect_chips()
    for i, dev in enumerate(devices):
        if dev.as_bh():
            bh_pci_idx.append(i)
            print(f"BH chip {i}...")

    BHChipReset().full_lds_reset(pci_interfaces=bh_pci_idx)


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0

"""
This file manages the register map for the device.
It loads the register map from yaml files, and provides functions to read and write registers.
"""

import os
import re
import yaml
import struct
import importlib.resources
from contextlib import contextmanager


@contextmanager
def package_data_file(package_relative_path):
    # 3.7 doesn't have importlib.resources.files & importlib.resources.as_file.
    with importlib.resources.path("tt_tools_common.data", "") as package_base_path:
        yield package_base_path / package_relative_path


INDEX_REGEXP = re.compile(r"\[\s*(\d*)\s*\]")
NAME_WITH_INDEX_REGEXP = re.compile(r"([^[\t ]*)\s*(\[\s*\d*\s*\])?")


def parse_indexed_register(name):
    m = re.match(NAME_WITH_INDEX_REGEXP, name)

    if m:
        index = None
        if m.group(2) is not None:
            index = int(re.match(INDEX_REGEXP, m.group(2)).group(1))
        ret_val = (m.group(1), index)
    else:
        ret_val = None
    return ret_val


def yaml_load(filepath):
    with open(filepath, "r") as file:
        return yaml.load(file, Loader=yaml.FullLoader)


class Register(object):
    # field_info is the Fields: part of the yaml
    def __init__(self, bit_count, fields, on_set=None, on_get=None):
        self.bit_count = bit_count
        self.fields = fields
        self.on_set_function = on_set
        self.on_get_function = on_get

    def _get_mask_and_shift(self, field_info):
        msb = field_info[1]
        lsb = field_info[2]
        mask = 0xFFFFFFFF >> (31 - (msb - lsb))
        return (mask, lsb)

    # Simple accessors
    def _set(self, data):
        self.data = data
        if self.on_set_function is not None:
            self.on_set_function(data)

    def _get(self):
        if not hasattr(self, "data"):
            raise RuntimeError(
                "Register has no data. Call _set() or one of the write functions."
            )
        if self.on_get_function is not None:
            return self.on_get_function()
        return self.data

    # Fields to write it dictionary of field_name -> value_to_write
    def write_fields(self, **kwargs):
        init = kwargs["__init"] if kwargs and "__init" in kwargs else None
        if init is not None:
            new_val = init
            total_mask = 0xFFFFFFFF  # TODO: this needs to be in terms of bit_count
        else:
            new_val = 0
            total_mask = 0

        for f in kwargs:
            if f == "__init":
                continue
            if f in self.fields:
                f_value = kwargs[f]
                field_info = self.fields[f]
                mask, shift = self._get_mask_and_shift(field_info)
                new_val = new_val & (~(mask << shift))  # Clear
                if f_value & mask != f_value:
                    raise RuntimeError(
                        f"Value written to field '{f}' is too big. The value is 0x{kwargs[f]:0x} which does not fit into mask 0x{mask:0x}."
                    )
                else:
                    new_val = new_val | (f_value << shift)
            else:
                raise RuntimeError(f"Register does not contain field '{f}'")

        if total_mask != 0xFFFFFFFF:
            raise RuntimeError(
                "If argument '__init' is not supplied, all register fields must be set. Alternativelly, use read-modify-write function (rmw_fields) which reads the register first to preserve untouched fields."
            )

        self._set(new_val)

    # Read-modify-write
    # Similar to write_fields, but does not assume that you set the whole value: it will initialize the value with what
    # is read from the register first.
    def rmw_fields(self, **kwargs):
        kwargs["__init"] = self._get()
        self.write_fields(**kwargs)

    # Returns a dictionary with all field names and values
    def read_fields(self):
        fields = {}
        for f in self.fields:
            field_info = self.fields[f]
            mask, shift = self._get_mask_and_shift(field_info)
            fields[f] = (self._get() >> shift) & mask
        return fields


class Registers(object):
    cached_registers = {}

    # Constructor:
    # - address_space_map_filename is the top-level yaml file that specifies register map
    # - reader_function takes 32-bit address and should return the value at that address
    # - writer_function takes 32-bit address, and 32-bit value: it should write the value to that address
    def __init__(self, address_space_map_filename, reader_function, writer_function):
        self.addr_space_map = {}
        self.set_access_functions(reader_function, writer_function)
        self.load_address_space_map(address_space_map_filename)

    # Sanity check for yaml file
    def sanity_check(self, loaded_yaml):
        if loaded_yaml is None:
            return
        for rname in loaded_yaml:
            if rname == "Regsize" or "Fields" not in loaded_yaml[rname]:
                continue
            r = loaded_yaml[rname]
            for fname in r["Fields"]:
                field_info = r["Fields"][fname]
                msb = field_info[1]
                lsb = field_info[2]
                if msb < lsb:
                    raise RuntimeError(
                        "In yaml for register %s: msb < lsb in fields %s"
                        % (rname, fname)
                    )

    def load_address_space_map(self, filename):
        self.filename = filename
        if filename in Registers.cached_registers:
            self.addr_space_map = Registers.cached_registers[filename]
            return
        with package_data_file(filename) as f:
            self.addr_space_map = yaml_load(f)
        for addr_space_name in self.addr_space_map:
            register_definition_filename = self.addr_space_map[addr_space_name][
                "filename"
            ]
            if "offset" not in self.addr_space_map[addr_space_name]:
                self.addr_space_map[addr_space_name]["offset"] = 0
            addr_space_offset = self.addr_space_map[addr_space_name]["offset"]
            # print ("Loading %s %s" % (register_definition_filename, addr_space_offset))
            # Load the file for the space
            regdef_filename = (
                os.path.dirname(filename) + "/" + register_definition_filename
            )
            self.addr_space_map[addr_space_name]["loaded_yaml"] = dict()
            # print ("Loading %s into addr_space %s " % (filename, addr_space_name))
            with package_data_file(regdef_filename) as f:
                loaded_yaml = yaml_load(f)

            if "Regsize" not in loaded_yaml:
                print(loaded_yaml)
                print(regdef_filename)
                raise RuntimeError(f"Regsize not in {regdef_filename}")

            Regsize = int(loaded_yaml["Regsize"])

            for reg in loaded_yaml:
                if reg != "Regsize":
                    loaded_yaml[reg]["Regsize"] = Regsize

            self.addr_space_map[addr_space_name]["loaded_yaml"] = loaded_yaml
            Registers.cached_registers[filename] = self.addr_space_map
            self.sanity_check(loaded_yaml)

    # Access functions work on 32 bit words
    def set_access_functions(self, read_function, write_function):
        self.read_function = read_function
        self.write_function = write_function

    def get_reg_space_addr(self, path):
        addr_space = self.addr_space_map[path]
        return addr_space["offset"]

    def get_path_info(self, path):
        """
        Given a path to a register (or register field), this function returns a tuple of:
        - addr:  32bit register address
        - mask:  covers all possible values for the field/register (0xffffffff for whole register). It is not shifted to the field position - use shift for that.
        - shift: for fields only (0 for whole register)
        - reg_info: full register definition with all fields, address ...
        """
        sections = path.split(".")
        addr_space_identifier = sections[0]

        reg_i = parse_indexed_register(addr_space_identifier)
        addr_space_identifier = reg_i[0]  # Discard the index, as it is not used here

        addr_space = self.addr_space_map[addr_space_identifier]

        if len(sections) == 1:  # This is only address space
            return addr_space["offset"]

        reg_name = sections[1]

        reg_i = parse_indexed_register(reg_name)
        reg_name = reg_i[0]
        array_index = int(reg_i[1]) if reg_i[1] is not None else 0

        if reg_name not in addr_space["loaded_yaml"]:
            assert False, f"Cannot find register {reg_name}"
        reg_info = addr_space["loaded_yaml"][reg_name]
        mask = self.get_mask_for_regsize(reg_info["Regsize"])
        shift = 0
        offset = 0

        if len(sections) > 2:  # There is a field
            field_info = reg_info["Fields"][sections[2]]
            mask, shift = self.get_mask_and_shift(field_info, reg_info["Regsize"])
            if len(field_info) > 4:  # There is an offset
                offset = field_info[3]

        addr = addr_space["offset"] + reg_info["Address"]
        if array_index > 0:
            if array_index >= reg_info["ArraySize"]:
                raise RuntimeError(
                    "Register in array %s has only %d elements, but path %s requested element with index %d"
                    % (reg_name, reg_info["ArraySize"], path, array_index)
                )

            addr = addr + array_index * reg_info["AddressIncrement"]
        addr += offset

        return (addr, mask, shift, reg_info)

    def get_mask_and_shift(self, field_info, regsize):
        msb = field_info[1]
        lsb = field_info[2]
        mask = self.get_mask_for_regsize(regsize) >> (regsize - 1 - (msb - lsb))
        return (mask, lsb)

    def set_write_delay_function(self, f):
        self.write_delay_f = f

    def set_write_delay(self, d):
        self.write_delay_f(d)

    def get_mask_for_regsize(self, regsize):
        if regsize == 32:
            return 0xFFFFFFFF
        elif regsize == 64:
            return 0xFFFFFFFFFFFFFFFF
        else:
            raise RuntimeError(f"Unsupported regsize in {self.filename}")

    # Given a dictionary 'fields' of field_name:value mappings, it sets the register with those values
    # - if 'init' is not provided, all fields must be supplied
    #   if 'init' is provided, the 32 bit value will be initialized to that before fields are written
    def write_fields(self, path, fields, init=None):
        addr, _, _, reg_info = self.get_path_info(path)
        Regsize = reg_info["Regsize"]
        assert Regsize <= 64, "read_fields not supported for Regsize > 64"
        FULL_REG_MASK = self.get_mask_for_regsize(Regsize)

        if init is not None:
            new_val = init
            total_mask = FULL_REG_MASK
        else:
            total_mask = 0
            new_val = 0

        for f in fields:
            if f not in reg_info["Fields"]:
                raise RuntimeError("Cannot find field %s in %s" % (f, path))
            else:
                field_info = reg_info["Fields"][f]
                mask, shift = self.get_mask_and_shift(field_info, reg_info["Regsize"])
                new_val = new_val & (~(mask << shift))  # Clear
                if fields[f] & mask != fields[f]:
                    raise RuntimeError(
                        "Value written to %s.%s is too big. The value is 0x%x which does not fit into mask 0x%x."
                        % (path, f, fields[f], mask)
                    )
                    return
                else:
                    new_val = new_val | (fields[f] << shift)
                    total_mask = total_mask | (mask << shift)

        if total_mask != FULL_REG_MASK:
            raise RuntimeError(
                f"If argument 'init' is not given, you must set all the fields in the register. Alternativelly, use read-modify-write function (rmw_fields) which reads the register first to preserve untouched fields. total_mask: {total_mask:0x} FULL_REG_MASK: {FULL_REG_MASK:0x}"
            )
            # raise RuntimeError ("Register has no data. Call _set() or one of the write functions.")
        if Regsize == 32:
            self.write_function(addr, new_val, path=path)
        else:
            self.write_function(addr, new_val & 0xFFFFFFFF, path=path)
            self.write_function(addr + 4, (new_val >> 32) & 0xFFFFFFFF, path=path)

    # Returns a dictionary with all field names and values
    def read_fields(self, path):
        fields = {}
        addr, _, _, reg_info = self.get_path_info(path)
        Regsize = reg_info["Regsize"]
        assert Regsize <= 64, "read_fields not supported for Regsize > 64"
        val = self.read_function(addr, path=path)
        if Regsize == 64:  # Read the upper 32 bits
            val = val | (self.read_function(addr + 4, path=path) << 32)
        for f in reg_info["Fields"]:
            field_info = reg_info["Fields"][f]
            mask, shift = self.get_mask_and_shift(field_info, reg_info["Regsize"])
            if len(field_info) > 4:  # Handle offsets
                val = self.read_function(addr + field_info[3], path=path)
            fields[f] = (val >> shift) & mask
        return fields

    # Read-modify-write
    # Similar to write_fields, but does not assume that you set the whole value: it will initialize the value with what
    # is read from the register first.
    # Returns the value of the register as it is read
    def rmw_fields(self, path, fields, init=None):
        addr, _, _, reg_info = self.get_path_info(path)
        if (
            len(next(iter(reg_info["Fields"].values()))) > 4
        ):  # Handle multi-register fields
            for f in fields:
                self.write32(f"{path}.{f}", fields[f])
        else:
            assert (
                reg_info["Regsize"] <= 64
            ), "rmw_fields not supported for Regsize > 64"
            current_reg_value = self.read32(path)
            if reg_info["Regsize"] == 64:  # Read the upper 32 bits
                current_reg_value = current_reg_value | (
                    self.read_function(addr + 4, path=path) << 32
                )
            self.write_fields(path, fields, init=current_reg_value)
            return current_reg_value

    def read32(self, path):
        addr, mask, shift, _ = self.get_path_info(path)
        val = self.read_function(addr, path=path)
        if type(val) is int:
            val = val >> shift  # Lose all lower bits
        else:
            raise RuntimeError("Expected integer value, but received %s" % str(val))
        val = val & mask
        return val

    def write32(self, path, data):
        addr, mask, shift, reg_info = self.get_path_info(path)
        if data & ~mask != 0:
            raise RuntimeError(
                "Value written to %s is too big. Value is 0x%x, Which does not fit into mask 0x%x."
                % (path, data, mask)
            )

        if mask != self.get_mask_for_regsize(
            reg_info["Regsize"]
        ):  # Need to read modify write
            current_val = self.read_function(addr, path=path)
            current_val = current_val & (~(mask << shift))  # Clear out the value
            new_val = current_val | (data << shift)
        else:
            new_val = data
        self.write_function(addr, new_val, path=path)

    def write_number_as_float32(self, path, num):
        # I don't want a duck, I want a float!!!!
        as_float32 = struct.pack("f", num)
        as_uint32 = struct.unpack("I", as_float32)
        self.write32(path, as_uint32[0])

    def read_number_as_float32(self, path):
        # I don't want a duck, I want a float!!!!
        val = self.read32(path)
        as_uint32 = struct.pack("I", val)
        as_float32 = struct.unpack("f", as_uint32)
        return as_float32[0]

    # set_value should already be shifted to match set_mask
    def rmw32(self, path, set_mask, set_value):
        assert ~set_mask & set_value == 0
        r = self.read32(path)
        self.write32(path, (r & ~set_mask) | set_value)

    # Gets only the address of a register
    def get_addr(self, path):
        addr, _, _, _ = self.get_path_info(path)
        return addr

    def WrNOC(self, addr, data, tlb):
        noc_addr = (tlb << 20) | (addr & 0xFFFFF)
        self.write_function(noc_addr, data, path="N/A")

    def RdNOC(self, addr, tlb):
        noc_addr = (tlb << 20) | (addr & 0xFFFFF)
        return self.read_function(noc_addr, path="N/A")

    # Finds all register paths that match the regexp
    def search(self, regexp, verbose=False):
        num_matches = 0
        printed_regs = (
            set()
        )  # So we only print the whole register once even if we mach multiple fields
        for addr_space_name in self.addr_space_map:
            addr_space = self.addr_space_map[addr_space_name]["loaded_yaml"]
            if addr_space is None:
                continue
            for reg_name in addr_space:
                reg = addr_space[reg_name]
                if reg_name != "Regsize" and "Fields" in reg:
                    for field_name in reg["Fields"]:
                        path = "%s.%s.%s" % (addr_space_name, reg_name, field_name)
                        m = re.search(regexp, path)
                        if m:
                            print("Matched '%s' in %s" % (m.group(0), path))
                            if verbose and reg_name not in printed_regs:
                                print("%s" % reg)
                                printed_regs.add(reg_name)
                            num_matches = num_matches + 1
        return num_matches

    def dump_scratch_regs(self):
        for i in range(0, 6):
            s = self.read32("ARC_RESET.SCRATCH[%d]" % i)
            print("  scratch[%d]: 0x%-8x %d " % (i, s, s))


def get_register_object(device) -> Registers:
    """
    Generate register object for a given device.
    This is used to read/write registers
    """
    if device.as_wh():
        raise Exception("Registers not supported for WH")

    def reader_function(byte_addr: int, path) -> int:
        return device.axi_read32(byte_addr)

    def writer_function(byte_addr: int, value: int, path) -> None:
        device.axi_write32(byte_addr, value)

    root_yaml_path = f"grayskull/axi-pci.yaml"
    return Registers(root_yaml_path, reader_function, writer_function)



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0
"""
Reset test for GS Tensix reset
"""
from pyluwen import detect_chips
from tt_tools_common.reset_common.gs_tensix_reset import GSTensixReset


def main():
    devices = detect_chips()
    for i, dev in enumerate(devices):
        if dev.as_gs():
            print(f"Resetting GS chip {i}...")
            GSTensixReset(dev).tensix_reset()


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: © 2024 Tenstorrent Inc.
# SPDX-License-Identifier: Apache-2.0
"""
Test mobo reset using reset config file
"""
from tt_tools_common.reset_common.galaxy_reset import GalaxyReset
from tt_tools_common.reset_common.wh_reset import WHChipReset
from tt_tools_common.reset_common.reset_utils import (
    parse_reset_input,
)

# UPDATE THIS BEFORE RUNNING
INPUT_FILE = "~/.config/tenstorrent/reset_config.json"
WH_RESET_PCI_NUM = 4


def main():
    # SAMPLE MOBO RESET CONFIG

    # mobo = "mobo-ce-44"
    # credo = ["6:0", "6:1", "7:0", "7:1"]
    # disabled_ports = [
    #     "0:0",
    #     "0:1",
    #     "0:2",
    #     "1:0",
    #     "1:1",
    #     "1:2",
    #     "6:0",
    #     "6:1",
    #     "6:2",
    #     "0:0",
    #     "0:1",
    #     "7:2",
    # ]

    # mobo_dict_list = [{"mobo": mobo, "credo": credo, "disabled_ports": disabled_ports}]
    mobo_dict_list = parse_reset_input(INPUT_FILE)["wh_mobo_reset"]
    mobo_reset_obj = GalaxyReset()
    mobo_reset_obj.warm_reset_mobo(mobo_dict_list)
    # reset_obj = WHChipReset()
    # reset_obj.full_lds_reset(list(range(WH_RESET_PCI_NUM)))


if __name__ == "__main__":
    main()
